@@ -1,12 +1,13 @@
 """Provides commands and queries for Git."""
+from __future__ import division, absolute_import, unicode_literals
+
 import re
-from cStringIO import StringIO
+from io import StringIO
 
 from cola import core
 from cola import gitcfg
 from cola import utils
 from cola import version
-from cola.compat import set
 from cola.git import git
 from cola.git import STDOUT
 from cola.i18n import N_
@@ -14,7 +15,7 @@ from cola.i18n import N_
 config = gitcfg.instance()
 
 
-class InvalidRepositoryError(StandardError):
+class InvalidRepositoryError(Exception):
     pass
 
 
@@ -137,7 +138,7 @@ def for_each_ref_basename(refs, git=git):
     out = git.for_each_ref(refs, format='%(refname)')[STDOUT]
     output = out.splitlines()
     non_heads = filter(lambda x: not x.endswith('/HEAD'), output)
-    return map(lambda x: x[len(refs) + 1:], non_heads)
+    return list(map(lambda x: x[len(refs) + 1:], non_heads))
 
 
 def all_refs(split=False, git=git):
@@ -181,9 +182,14 @@ def tracked_branch(branch=None, config=None):
     return None
 
 
-def untracked_files(git=git):
+def untracked_files(git=git, paths=None):
     """Returns a sorted list of untracked files."""
-    out = git.ls_files(z=True, others=True, exclude_standard=True)[STDOUT]
+
+    if paths is None:
+        paths = []
+    filter_paths = ['--'] + paths
+    out = git.ls_files(z=True, others=True, exclude_standard=True,
+                       *filter_paths)[STDOUT]
     if out:
         return out[:-1].split('\0')
     return []
@@ -191,9 +197,7 @@ def untracked_files(git=git):
 
 def tag_list():
     """Return a list of tags."""
-    tags = for_each_ref_basename('refs/tags')
-    tags.reverse()
-    return tags
+    return list(reversed(for_each_ref_basename('refs/tags')))
 
 
 def log(git, *args, **kwargs):
@@ -227,15 +231,35 @@ def common_diff_opts(config=config):
     return opts
 
 
-def sha1_diff(git, sha1):
-    return git.diff(sha1+'^!', **common_diff_opts())[STDOUT]
+def _add_filename(args, filename):
+    if filename:
+        args.extend(['--', filename])
 
 
-def diff_info(sha1, git=git):
+def sha1_diff(git, sha1, filename=None):
+    """Return the diff for a sha1"""
+    # Naively "$sha1^!" is what we'd like to use but that doesn't
+    # give the correct result for merges--the diff is reversed.
+    # Be explicit and compare sha1 against its first parent.
+    args = [sha1 + '~', sha1]
+    opts = common_diff_opts()
+    _add_filename(args, filename)
+    status, out, err = git.diff(*args, **opts)
+    if status != 0:
+        # We probably don't have "$sha1~" because this is the root commit.
+        # "git show" is clever enough to handle the root commit.
+        args = [sha1 + '^!']
+        _add_filename(args, filename)
+        status, out, err = git.show(pretty='format:', *args, **opts)
+        out = out.lstrip()
+    return out
+
+
+def diff_info(sha1, git=git, filename=None):
     decoded = log(git, '-1', sha1, '--', pretty='format:%b').strip()
     if decoded:
         decoded += '\n\n'
-    return decoded + sha1_diff(git, sha1)
+    return decoded + sha1_diff(git, sha1, filename=filename)
 
 
 def diff_helper(commit=None,
@@ -292,7 +316,6 @@ def diff_helper(commit=None,
 
 def extract_diff_header(status, deleted,
                         with_diff_header, suppress_header, diffoutput):
-    encode = core.encode
     headers = []
 
     if diffoutput.startswith('Submodule'):
@@ -310,14 +333,14 @@ def extract_diff_header(status, deleted,
         if not start and '@@' == line[:2] and '@@' in line[2:]:
             start = True
         if start or (deleted and del_tag in line):
-            output.write(encode(line) + '\n')
+            output.write(line + '\n')
         else:
             if with_diff_header:
                 headers.append(line)
             elif not suppress_header:
-                output.write(encode(line) + '\n')
+                output.write(line + '\n')
 
-    result = core.decode(output.getvalue())
+    result = output.getvalue()
     output.close()
 
     if with_diff_header:
@@ -411,7 +434,10 @@ def worktree_state(head='HEAD'):
            state.get('upstream_changed', []))
 
 
-def worktree_state_dict(head='HEAD', update_index=False, display_untracked=True):
+def worktree_state_dict(head='HEAD',
+                        update_index=False,
+                        display_untracked=True,
+                        paths=None):
     """Return a dict of files in various states of being
 
     :rtype: dict, keys are staged, unstaged, untracked, unmerged,
@@ -421,9 +447,9 @@ def worktree_state_dict(head='HEAD', update_index=False, display_untracked=True)
     if update_index:
         git.update_index(refresh=True)
 
-    staged, unmerged, staged_submods = diff_index(head)
-    modified, modified_submods = diff_worktree()
-    untracked = display_untracked and untracked_files() or []
+    staged, unmerged, staged_submods = diff_index(head, paths=paths)
+    modified, modified_submods = diff_worktree(paths)
+    untracked = display_untracked and untracked_files(paths=paths) or []
 
     # Remove unmerged paths from the modified list
     unmerged_set = set(unmerged)
@@ -462,12 +488,16 @@ def worktree_state_dict(head='HEAD', update_index=False, display_untracked=True)
             'submodules': submodules}
 
 
-def diff_index(head, cached=True):
+def diff_index(head, cached=True, paths=None):
     submodules = set()
     staged = []
     unmerged = []
 
-    status, out, err = git.diff_index(head, '--', cached=cached, z=True)
+    if paths is None:
+        paths = []
+    filter_paths = [head, '--'] + paths
+    status, out, err = git.diff_index(cached=cached, z=True,
+                                      *filter_paths)
     if status != 0:
         # handle git init
         return all_files(), unmerged, submodules
@@ -486,11 +516,14 @@ def diff_index(head, cached=True):
     return staged, unmerged, submodules
 
 
-def diff_worktree():
+def diff_worktree(paths=None):
     modified = []
     submodules = set()
 
-    status, out, err = git.diff_files(z=True)
+    if paths is None:
+        paths = []
+    filter_paths = ['--'] + paths
+    status, out, err = git.diff_files(z=True, *filter_paths)
     if status != 0:
         # handle git init
         out = git.ls_files(modified=True, z=True)[STDOUT]
