@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 import os
 import subprocess
 from cola import core
@@ -61,6 +63,7 @@ class Interaction(object):
     def confirm_config_action(cls, name, opts):
         return cls.confirm(N_('Run %s?') % name,
                            N_('Run the "%s" command?') % name,
+                           '',
                            ok_text=N_('Run'))
 
     @classmethod
