@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 import subprocess
 
 from cola import core
@@ -71,7 +73,7 @@ class DAG(Observable):
         if self.set_count(args.count):
             self.overrides['count'] = args.count
 
-        if args.args:
+        if hasattr(args, 'args') and args.args:
             ref = subprocess.list2cmdline(map(core.decode, args.args))
             if self.set_ref(ref):
                 self.overrides['ref'] = ref
@@ -242,6 +244,8 @@ class RepoReader(object):
             self._topo_list.append(c)
             return c
 
+    __next__ = next # for Python 3
+
     def __getitem__(self, sha1):
         return self._objects[sha1]
 
