@@ -1,6 +1,7 @@
 # Copyright (c) 2008 David Aguilar
 """This module provides the central cola model.
 """
+from __future__ import division, absolute_import, unicode_literals
 
 import os
 import copy
@@ -9,12 +10,12 @@ from cola import core
 from cola import git
 from cola import gitcfg
 from cola import gitcmds
-from cola.compat import set
 from cola.git import STDOUT
 from cola.observable import Observable
 from cola.decorators import memoize
 from cola.models.selection import selection_model
 from cola.models import prefs
+from cola.compat import ustr
 
 
 # Static GitConfig instance
@@ -78,6 +79,7 @@ class MainModel(Observable):
         self.directory = ''
         self.project = ''
         self.remotes = []
+        self.filter_paths = None
 
         self.commitmsg = ''
         self.modified = []
@@ -123,7 +125,10 @@ class MainModel(Observable):
 
     def save_commitmsg(self, msg):
         path = self.git.git_path('GIT_COLA_MSG')
-        core.write(path, msg)
+        try:
+            core.write(path, msg)
+        except:
+            pass
 
     def set_diff_text(self, txt):
         self.diff_text = txt
@@ -163,6 +168,10 @@ class MainModel(Observable):
         return self.git.log('-1', no_color=True, pretty='format:%s%n%n%b',
                             *args)[STDOUT]
 
+    def update_path_filter(self, filter_paths):
+        self.filter_paths = filter_paths
+        self.update_file_status()
+
     def update_file_status(self, update_index=False):
         self.notify_observers(self.message_about_to_update)
         self._update_files(update_index=update_index)
@@ -173,7 +182,7 @@ class MainModel(Observable):
         self.notify_observers(self.message_about_to_update)
         self._update_merge_rebase_status()
         self._update_files(update_index=update_index)
-        self._update_refs()
+        self._update_remotes()
         self._update_branches_and_tags()
         self._update_branch_heads()
         self.notify_observers(self.message_updated)
@@ -182,7 +191,8 @@ class MainModel(Observable):
         display_untracked = prefs.display_untracked()
         state = gitcmds.worktree_state_dict(head=self.head,
                                             update_index=update_index,
-                                            display_untracked=display_untracked)
+                                            display_untracked=display_untracked,
+                                            paths=self.filter_paths)
         self.staged = state.get('staged', [])
         self.modified = state.get('modified', [])
         self.unmerged = state.get('unmerged', [])
@@ -202,7 +212,7 @@ class MainModel(Observable):
         return not(bool(self.staged or self.modified or
                         self.unmerged or self.untracked))
 
-    def _update_refs(self):
+    def _update_remotes(self):
         self.remotes = self.git.remote()[STDOUT].splitlines()
 
     def _update_branch_heads(self):
@@ -221,6 +231,10 @@ class MainModel(Observable):
         if self.is_merging and self.mode == self.mode_amend:
             self.set_mode(self.mode_none)
 
+    def update_remotes(self):
+        self._update_remotes()
+        self._update_branches_and_tags()
+
     def delete_branch(self, branch):
         return self.git.branch(branch, D=True)
 
@@ -254,7 +268,7 @@ class MainModel(Observable):
 
         max_arg_len = 32 * 4 * 1024
         avg_filename_len = 300
-        size = max_arg_len / avg_filename_len
+        size = max_arg_len // avg_filename_len
 
         status = 0
         outs = []
@@ -303,7 +317,7 @@ class MainModel(Observable):
 
     def config_set(self, key, value, local=True):
         # git config category.key value
-        strval = unicode(value)
+        strval = ustr(value)
         if type(value) is bool:
             # git uses "true" and "false"
             strval = strval.lower()
@@ -338,20 +352,6 @@ class MainModel(Observable):
             newdict[k]=v
         return newdict
 
-    def commit_with_msg(self, msg, tmpfile, amend=False):
-        """Creates a git commit."""
-
-        if not msg.endswith('\n'):
-            msg += '\n'
-
-        # Create the commit message file
-        core.write(tmpfile, msg)
-
-        # Run 'git commit'
-        status, out, err = self.git.commit(F=tmpfile, v=True, amend=amend)
-        core.unlink(tmpfile)
-        return (status, out, err)
-
     def remote_url(self, name, action):
         if action == 'push':
             url = self.git.config('remote.%s.pushurl' % name,
@@ -366,18 +366,17 @@ class MainModel(Observable):
                     ffwd=True,
                     tags=False,
                     rebase=False,
+                    pull=False,
                     push=False):
-        # Swap the branches in push mode (reverse of fetch)
         if push:
-            tmp = local_branch
-            local_branch = remote_branch
-            remote_branch = tmp
+            # Swap the branches in push mode (reverse of fetch)
+            local_branch, remote_branch = remote_branch, local_branch
         if ffwd:
             branch_arg = '%s:%s' % (remote_branch, local_branch)
         else:
             branch_arg = '+%s:%s' % (remote_branch, local_branch)
         args = [remote]
-        if local_branch and remote_branch:
+        if local_branch and remote_branch and not pull:
             args.append(branch_arg)
         elif local_branch:
             args.append(local_branch)
@@ -401,7 +400,7 @@ class MainModel(Observable):
         return self.run_remote_action(self.git.push, remote, push=True, **opts)
 
     def pull(self, remote, **opts):
-        return self.run_remote_action(self.git.pull, remote, push=True, **opts)
+        return self.run_remote_action(self.git.pull, remote, pull=True, **opts)
 
     def create_branch(self, name, base, track=False, force=False):
         """Create a branch named 'name' from revision 'base'
@@ -488,7 +487,7 @@ class MainModel(Observable):
         return status, out, err
 
     def getcwd(self):
-        """If we've chosen a directory then use it, otherwise os.getcwd()."""
+        """If we've chosen a directory then use it, otherwise use current"""
         if self.directory:
             return self.directory
         return core.getcwd()
