@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 from cola import gitcfg
 from cola import observable
 
@@ -32,7 +34,7 @@ def display_untracked():
 
 def editor():
     app = config().get(EDITOR, 'gvim')
-    return {'vim': 'gvim'}.get(app, app)
+    return {'vim': 'gvim -f'}.get(app, app)
 
 
 def history_browser():
