@@ -62,11 +62,17 @@ def create_listwidget_item(text, filename):
 
 class TreeWidgetItem(QtGui.QTreeWidgetItem):
 
-    def __init__(self, text, filename, exists):
+    TYPE = QtGui.QStandardItem.UserType + 101
+
+    def __init__(self, path, icon, exists):
         QtGui.QTreeWidgetItem.__init__(self)
+        self.path = path
         self.exists = exists
-        self.setIcon(0, cached_icon_from_path(filename))
-        self.setText(0, text)
+        self.setIcon(0, cached_icon_from_path(icon))
+        self.setText(0, path)
+
+    def type(self):
+        return self.TYPE
 
 
 def create_treewidget_item(text, filename, exists=True):
@@ -74,6 +80,24 @@ def create_treewidget_item(text, filename, exists=True):
     return TreeWidgetItem(text, filename, exists)
 
 
+def paths_from_indexes(model, indexes,
+                       item_type=TreeWidgetItem.TYPE,
+                       item_filter=None):
+    """Return paths from a list of QStandardItemModel indexes"""
+    items = [model.itemFromIndex(i) for i in indexes]
+    return paths_from_items(items, item_type=item_type, item_filter=item_filter)
+
+
+def paths_from_items(items,
+                     item_type=TreeWidgetItem.TYPE,
+                     item_filter=None):
+    """Return a list of paths from a list of items"""
+    if item_filter is None:
+        item_filter = lambda x: True
+    return [i.path for i in items
+            if i.type() == item_type and item_filter(i)]
+
+
 @memoize
 def cached_icon_from_path(filename):
     return QtGui.QIcon(filename)
@@ -279,8 +303,8 @@ def opendir_dialog(title, path):
     flags = (QtGui.QFileDialog.ShowDirsOnly |
              QtGui.QFileDialog.DontResolveSymlinks)
     return ustr(QtGui.QFileDialog
-                        .getExistingDirectory(active_window(),
-                                              title, path, flags))
+                     .getExistingDirectory(active_window(),
+                                           title, path, flags))
 
 
 def save_as(filename, title='Save As...'):
@@ -294,6 +318,13 @@ def icon(basename):
     return QtGui.QIcon(resources.icon(basename))
 
 
+def copy_path(filename):
+        """Copy a filename prefixed by current directory to the clipboard"""
+        if filename is not None:
+            curdir = core.getcwd()
+            set_clipboard(os.path.join(curdir, filename))
+
+
 def set_clipboard(text):
     """Sets the copy/paste buffer to text."""
     if not text:
@@ -450,6 +481,11 @@ def options_icon():
     return icon('options.svg')
 
 
+def filter_icon():
+    """Return a filter icon"""
+    return icon('view-filter.png')
+
+
 def dir_close_icon():
     """Return a standard closed directory icon"""
     return cached_icon(QtGui.QStyle.SP_DirClosedIcon)
@@ -547,7 +583,7 @@ def create_button(text='', layout=None, tooltip=None, icon=None):
     button.setCursor(Qt.PointingHandCursor)
     if text:
         button.setText(text)
-    if icon:
+    if icon is not None:
         button.setIcon(icon)
     if tooltip is not None:
         button.setToolTip(tooltip)
@@ -592,7 +628,7 @@ class DockTitleBarWidget(QtGui.QWidget):
         QtGui.QWidget.__init__(self, parent)
         self.label = label = QtGui.QLabel()
         font = label.font()
-        font.setCapitalization(QtGui.QFont.SmallCaps)
+        font.setBold(True)
         label.setFont(font)
         label.setText(title)
 
@@ -682,6 +718,13 @@ def create_toolbutton(text=None, layout=None, tooltip=None, icon=None):
     return button
 
 
+def mimedata_from_paths(paths):
+    """Return mimedata with a list of absolute path URLs"""
+    urls = [QtCore.QUrl(core.abspath(path)) for path in paths]
+    mimedata = QtCore.QMimeData()
+    mimedata.setUrls(urls)
+    return mimedata
+
 # Syntax highlighting
 
 def TERMINAL(pattern):
