@@ -1,5 +1,6 @@
 """Text wrapping and filling.
 """
+from __future__ import division, absolute_import, unicode_literals
 
 # Copyright (C) 1999-2001 Gregory P. Ward.
 # Copyright (C) 2002, 2003 Python Software Foundation.
@@ -9,6 +10,8 @@
 
 import re
 
+from cola.compat import ustr
+
 
 class TextWrapper(object):
     """
@@ -80,7 +83,7 @@ class TextWrapper(object):
           'use', ' ', 'the', ' ', '-b', ' ', option!'
         otherwise.
         """
-        if isinstance(text, unicode):
+        if isinstance(text, ustr):
             if self.break_on_hyphens:
                 pat = self.wordsep_re_uni
             else:
@@ -91,7 +94,7 @@ class TextWrapper(object):
             else:
                 pat = self.wordsep_simple_re
         chunks = pat.split(text)
-        chunks = filter(None, chunks)  # remove empty chunks
+        chunks = list(filter(None, chunks))  # remove empty chunks
         return chunks
 
     def _wrap_chunks(self, chunks):
@@ -110,7 +113,7 @@ class TextWrapper(object):
 
         # Arrange in reverse order so items can be efficiently popped
         # from a stack of chucks.
-        chunks.reverse()
+        chunks = list(reversed(chunks))
 
         while chunks:
 
@@ -200,21 +203,56 @@ def word_wrap(text, tabwidth, limit):
 
     >>> text = 'a bb ccc dddd\neeeee'
     >>> word_wrap(text, 8, 2)
-    'a\nbb\nccc\ndddd\neeeee'
+    u'a\nbb\nccc\ndddd\neeeee'
 
     >>> word_wrap(text, 8, 4)
-    'a bb\nccc\ndddd\neeeee'
+    u'a bb\nccc\ndddd\neeeee'
 
     >>> text = 'a bb ccc dddd\n\teeeee'
     >>> word_wrap(text, 8, 4)
-    'a bb\nccc\ndddd\n\t\neeeee'
+    u'a bb\nccc\ndddd\n\t\neeeee'
 
     """
 
     lines = []
 
     # Acked-by:, Signed-off-by:, Helped-by:, etc.
-    special_tag_rgx = re.compile('^[a-zA-Z_-]+:')
+    special_tag_rgx = re.compile(
+            r'^('
+            r'Acked-by|'
+            r"Ack'd-by|"
+            r'Based-on-patch-by|'
+            r'Cheered-on-by|'
+            r'Co-authored-by|'
+            r'Comments-by|'
+            r'Confirmed-by|'
+            r'Contributions-by|'
+            r'Debugged-by|'
+            r'Discovered-by|'
+            r'Explained-by|'
+            r'Backtraced-by|'
+            r'Helped-by|'
+            r'Liked-by|'
+            r'Improved-by|'
+            r'Inspired-by|'
+            r'Initial-patch-by|'
+            r'Noticed-by|'
+            r'Original-patch-by|'
+            r'Originally-by|'
+            r'Mentored-by|'
+            r'Patch-by|'
+            r'Proposed-by|'
+            r'Reported-by|'
+            r'Requested-by|'
+            r'Reviewed-by|'
+            r'Signed-off-by|'
+            r'Signed-Off-by|'
+            r'Spotted-by|'
+            r'Suggested-by|'
+            r'Tested-by|'
+            r'Tested-on-([a-zA-Z-_]+)-by|'
+            r'With-suggestions-by'
+            r'):')
 
     w = TextWrapper(width=limit,
                     tabwidth=tabwidth,
