@@ -1,4 +1,4 @@
-import os
+from __future__ import division, absolute_import, unicode_literals
 
 from PyQt4 import QtGui
 from PyQt4 import QtCore
@@ -12,7 +12,6 @@ from cola import gitcmds
 from cola import utils
 from cola import qtutils
 from cola.cmds import BaseCommand
-from cola.compat import set
 from cola.git import git
 from cola.i18n import N_
 from cola.interaction import Interaction
@@ -25,6 +24,7 @@ from cola.models.selection import selection_model
 from cola.widgets import defs
 from cola.widgets import standard
 from cola.widgets.selectcommits import select_commits
+from cola.compat import ustr
 
 
 def worktree_browser_widget(parent, update=True):
@@ -48,7 +48,7 @@ class Browser(standard.Widget):
         self.tree = RepoTreeView(self)
         self.mainlayout = QtGui.QHBoxLayout()
         self.setLayout(self.mainlayout)
-        self.mainlayout.setMargin(0)
+        self.mainlayout.setMargin(defs.no_margin)
         self.mainlayout.setSpacing(defs.spacing)
         self.mainlayout.addWidget(self.tree)
         self.resize(720, 420)
@@ -69,15 +69,15 @@ class Browser(standard.Widget):
 
     def _updated_callback(self):
         branch = self.model.currentbranch
-        curdir = os.getcwd()
+        curdir = core.getcwd()
         msg = N_('Repository: %s') % curdir
         msg += '\n'
         msg += N_('Branch: %s') % branch
         self.setToolTip(msg)
 
-        title = N_('%s: %s - Browse') % (self.model.project, branch)
+        title = N_('%(project)s: %(branch)s - Browse') % dict(project=self.model.project, branch=branch)
         if self.mode == self.model.mode_amend:
-            title += ' (%s)' % N_('Amending')
+            title += ' %s' % N_('(Amending)')
         self.setWindowTitle(title)
 
 
@@ -87,13 +87,16 @@ class RepoTreeView(standard.TreeView):
     def __init__(self, parent):
         standard.TreeView.__init__(self, parent)
 
+        self.setDragEnabled(True)
         self.setRootIsDecorated(True)
         self.setSortingEnabled(False)
         self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
 
         # Observe model updates
         model = main.model()
-        model.add_observer(model.message_updated, self.update_actions)
+        model.add_observer(model.message_updated, self.emit_update)
+
+        self.connect(self, SIGNAL('update()'), self.update_actions)
 
         # The non-Qt cola application model
         self.connect(self, SIGNAL('expanded(QModelIndex)'), self.size_columns)
@@ -130,6 +133,7 @@ class RepoTreeView(standard.TreeView):
                                     N_('Launch git-difftool on the current path.'),
                                     cmds.run(cmds.LaunchDifftool),
                                     cmds.LaunchDifftool.SHORTCUT)
+
         self.action_difftool_predecessor =\
                 self._create_action(N_('Diff Against Predecessor...'),
                                     N_('Launch git-difftool against previous versions.'),
@@ -144,12 +148,15 @@ class RepoTreeView(standard.TreeView):
                 self._create_action(cmds.LaunchEditor.name(),
                                     N_('Edit selected path(s).'),
                                     cmds.run(cmds.LaunchEditor),
-                                    cmds.LaunchDifftool.SHORTCUT)
+                                    cmds.LaunchEditor.SHORTCUT)
 
     def size_columns(self):
         """Set the column widths."""
         self.resizeColumnToContents(0)
 
+    def emit_update(self):
+        self.emit(SIGNAL('update()'))
+
     def update_actions(self):
         """Enable/disable actions."""
         selection = self.selected_paths()
@@ -199,10 +206,10 @@ class RepoTreeView(standard.TreeView):
 
         paths = self.selected_paths()
         model = main.model()
-        model_staged = utils.add_parents(set(model.staged))
-        model_modified = utils.add_parents(set(model.modified))
-        model_unmerged = utils.add_parents(set(model.unmerged))
-        model_untracked = utils.add_parents(set(model.untracked))
+        model_staged = utils.add_parents(model.staged)
+        model_modified = utils.add_parents(model.modified)
+        model_unmerged = utils.add_parents(model.unmerged)
+        model_untracked = utils.add_parents(model.untracked)
 
         for path in paths:
             if path in model_unmerged:
@@ -240,17 +247,19 @@ class RepoTreeView(standard.TreeView):
         index = model_index.sibling(model_index.row(), 0)
         return self.model().itemFromIndex(index)
 
+    def paths_from_indexes(self, indexes):
+        return qtutils.paths_from_indexes(self.model(), indexes,
+                                          item_type=GitRepoNameItem.TYPE)
+
     def selected_paths(self):
         """Return the selected paths."""
-        items = map(self.model().itemFromIndex, self.selectedIndexes())
-        return [i.path for i in items
-                    if i.type() == GitRepoNameItem.TYPE]
+        return self.paths_from_indexes(self.selectedIndexes())
 
     def selected_staged_paths(self, selection=None):
         """Return selected staged paths."""
         if not selection:
             selection = self.selected_paths()
-        staged = utils.add_parents(set(main.model().staged))
+        staged = utils.add_parents(main.model().staged)
         return [p for p in selection if p in staged]
 
     def selected_modified_paths(self, selection=None):
@@ -258,7 +267,7 @@ class RepoTreeView(standard.TreeView):
         if not selection:
             selection = self.selected_paths()
         model = main.model()
-        modified = utils.add_parents(set(model.modified))
+        modified = utils.add_parents(model.modified)
         return [p for p in selection if p in modified]
 
     def selected_unstaged_paths(self, selection=None):
@@ -266,8 +275,8 @@ class RepoTreeView(standard.TreeView):
         if not selection:
             selection = self.selected_paths()
         model = main.model()
-        modified = utils.add_parents(set(model.modified))
-        untracked = utils.add_parents(set(model.untracked))
+        modified = utils.add_parents(model.modified)
+        untracked = utils.add_parents(model.untracked)
         unstaged = modified.union(untracked)
         return [p for p in selection if p in unstaged]
 
@@ -278,7 +287,7 @@ class RepoTreeView(standard.TreeView):
         model = main.model()
         staged = set(self.selected_staged_paths())
         modified = set(self.selected_modified_paths())
-        untracked = utils.add_parents(set(model.untracked))
+        untracked = utils.add_parents(model.untracked)
         tracked = staged.union(modified)
         return [p for p in selection
                 if p not in untracked or p in tracked]
@@ -352,7 +361,7 @@ class BrowserController(QtCore.QObject):
 
     def view_history(self, entries):
         """Launch the configured history browser path-limited to entries."""
-        entries = map(unicode, entries)
+        entries = list(map(ustr, entries))
         cmds.do(cmds.VisualizePaths, entries)
 
     def query_model(self, model_index):
@@ -364,7 +373,7 @@ class BrowserController(QtCore.QObject):
         self.updated.add(path)
         GitRepoEntryManager.entry(path).update()
         entry = GitRepoEntryManager.entry
-        for row in xrange(item.rowCount()):
+        for row in range(item.rowCount()):
             path = item.child(row, 0).path
             entry(path).update()
 
@@ -389,6 +398,7 @@ class BrowseModel(object):
 
 class SaveBlob(BaseCommand):
     def __init__(self, model):
+        BaseCommand.__init__(self)
         self.model = model
 
     def do(self):
@@ -422,7 +432,7 @@ class BrowseDialog(QtGui.QDialog):
         dlg.setModel(dlg_model)
         dlg.setWindowTitle(N_('Browsing %s') % model.ref)
         if hasattr(parent, 'width'):
-            dlg.resize(parent.width()*3/4, 333)
+            dlg.resize(parent.width()*3//4, 333)
         else:
             dlg.resize(420, 333)
         dlg.show()
@@ -439,7 +449,7 @@ class BrowseDialog(QtGui.QDialog):
         dlg_model = GitTreeModel(ref, dlg)
         dlg.setModel(dlg_model)
         dlg.setWindowTitle(N_('Select file from "%s"') % model.ref)
-        dlg.resize(parent.width()*3/4, 333)
+        dlg.resize(parent.width()*3//4, 333)
         dlg.show()
         dlg.raise_()
         if dlg.exec_() != dlg.Accepted:
@@ -456,7 +466,7 @@ class BrowseDialog(QtGui.QDialog):
         dlg.setModel(dlg_model)
         dlg.expandAll()
         dlg.setWindowTitle(title)
-        dlg.resize(parent.width()*3/4, 333)
+        dlg.resize(parent.width()*3//4, 333)
         dlg.show()
         dlg.raise_()
         if dlg.exec_() != dlg.Accepted:
@@ -466,13 +476,14 @@ class BrowseDialog(QtGui.QDialog):
     def __init__(self, model, select_file=False, parent=None):
         QtGui.QDialog.__init__(self, parent)
         self.setAttribute(Qt.WA_MacMetalStyle)
-        self.setWindowModality(Qt.WindowModal)
+        if parent is not None:
+            self.setWindowModality(Qt.WindowModal)
 
         # updated for use by commands
         self.model = model
 
         # widgets
-        self.tree = GitTreeWidget(model.ref, parent=self)
+        self.tree = GitTreeWidget(parent=self)
         self.close = QtGui.QPushButton(N_('Close'))
         self.save = QtGui.QPushButton(select_file and N_('Select') or N_('Save'))
         self.save.setDefault(True)
@@ -542,7 +553,7 @@ class BrowseDialog(QtGui.QDialog):
 
 
 class GitTreeWidget(standard.TreeView):
-    def __init__(self, ref, parent=None):
+    def __init__(self, parent=None):
         standard.TreeView.__init__(self, parent)
         self.setHeaderHidden(True)
 
