@@ -1,6 +1,7 @@
+from __future__ import division, absolute_import, unicode_literals
+
 import collections
 import math
-import sys
 
 from PyQt4 import QtGui
 from PyQt4 import QtCore
@@ -26,9 +27,11 @@ from cola.widgets.standard import MainWindow
 from cola.widgets.standard import TreeWidget
 from cola.widgets.diff import COMMITS_SELECTED
 from cola.widgets.diff import DiffWidget
+from cola.widgets.filelist import FileWidget
+from cola.compat import ustr
 
 
-def git_dag(model, args=None):
+def git_dag(model, args=None, settings=None):
     """Return a pre-populated git DAG widget."""
     branch = model.currentbranch
     # disambiguate between branch names and filenames by using '--'
@@ -36,7 +39,7 @@ def git_dag(model, args=None):
     dag = DAG(branch_doubledash, 1000)
     dag.set_arguments(args)
 
-    view = DAGView(model, dag, None)
+    view = GitDAG(model, dag, settings=settings)
     if dag.ref:
         view.display()
     return view
@@ -102,7 +105,7 @@ class ViewerMixin(object):
         if sha1 is None:
             return
         short_sha1 = sha1[:7]
-        GitArchiveDialog.save(sha1, short_sha1, self)
+        GitArchiveDialog.save_hashed_objects(sha1, short_sha1, self)
 
     def save_blob_dialog(self):
         sha1 = self.selected_sha1()
@@ -273,8 +276,8 @@ class CommitTreeWidget(ViewerMixin, TreeWidget):
 
     def adjust_columns(self):
         width = self.width()-20
-        zero = width*2/3
-        onetwo = width/6
+        zero = width*2//3
+        onetwo = width//6
         self.setColumnWidth(0, zero)
         self.setColumnWidth(1, onetwo)
         self.setColumnWidth(2, onetwo)
@@ -299,8 +302,7 @@ class CommitTreeWidget(ViewerMixin, TreeWidget):
         items = self.selectedItems()
         if not items:
             return
-        items.reverse()
-        sha1s = [item.commit.sha1 for item in items]
+        sha1s = [item.commit.sha1 for item in reversed(items)]
         all_sha1s = [c.sha1 for c in self.commits]
         cmds.do(cmds.FormatPatch, sha1s, all_sha1s)
 
@@ -315,19 +317,20 @@ class CommitTreeWidget(ViewerMixin, TreeWidget):
         QtGui.QTreeWidget.mousePressEvent(self, event)
 
 
-class DAGView(MainWindow):
+class GitDAG(MainWindow):
     """The git-dag widget."""
 
-    def __init__(self, model, dag, parent=None, args=None):
+    def __init__(self, model, dag, parent=None, settings=None):
         MainWindow.__init__(self, parent)
 
         self.setAttribute(Qt.WA_MacMetalStyle)
         self.setMinimumSize(420, 420)
 
         # change when widgets are added/removed
-        self.widget_version = 1
+        self.widget_version = 2
         self.model = model
         self.dag = dag
+        self.settings = settings
 
         self.commits = {}
         self.commit_list = []
@@ -345,13 +348,16 @@ class DAGView(MainWindow):
         self.maxresults.setSuffix('')
 
         self.zoom_out = qtutils.create_action_button(
-                N_('Zoom Out'), qtutils.theme_icon('zoom-out.png'))
+                tooltip=N_('Zoom Out'),
+                icon=qtutils.theme_icon('zoom-out.png'))
 
         self.zoom_in = qtutils.create_action_button(
-                N_('Zoom In'), qtutils.theme_icon('zoom-in.png'))
+                tooltip=N_('Zoom In'),
+                icon=qtutils.theme_icon('zoom-in.png'))
 
         self.zoom_to_fit = qtutils.create_action_button(
-                N_('Zoom to Fit'), qtutils.theme_icon('zoom-fit-best.png'))
+                tooltip=N_('Zoom to Fit'),
+                icon=qtutils.theme_icon('zoom-fit-best.png'))
 
         self.notifier = notifier = observable.Observable()
         self.notifier.refs_updated = refs_updated = 'refs_updated'
@@ -359,6 +365,7 @@ class DAGView(MainWindow):
 
         self.treewidget = CommitTreeWidget(notifier, self)
         self.diffwidget = DiffWidget(notifier, self)
+        self.filewidget = FileWidget(notifier, self)
         self.graphview = GraphView(notifier, self)
 
         self.controls_layout = QtGui.QHBoxLayout()
@@ -375,6 +382,9 @@ class DAGView(MainWindow):
         log_dock_titlebar = self.log_dock.titleBarWidget()
         log_dock_titlebar.add_corner_widget(self.controls_widget)
 
+        self.file_dock = qtutils.create_dock(N_('Files'), self)
+        self.file_dock.setWidget(self.filewidget)
+
         self.diff_dock = qtutils.create_dock(N_('Diff'), self)
         self.diff_dock.setWidget(self.diffwidget)
 
@@ -404,6 +414,7 @@ class DAGView(MainWindow):
         self.view_menu.addAction(self.log_dock.toggleViewAction())
         self.view_menu.addAction(self.graphview_dock.toggleViewAction())
         self.view_menu.addAction(self.diff_dock.toggleViewAction())
+        self.view_menu.addAction(self.file_dock.toggleViewAction())
         self.view_menu.addSeparator()
         self.view_menu.addAction(self.lock_layout_action)
 
@@ -412,10 +423,10 @@ class DAGView(MainWindow):
 
         left = Qt.LeftDockWidgetArea
         right = Qt.RightDockWidgetArea
-        bottom = Qt.BottomDockWidgetArea
         self.addDockWidget(left, self.log_dock)
+        self.addDockWidget(left, self.diff_dock)
         self.addDockWidget(right, self.graphview_dock)
-        self.addDockWidget(bottom, self.diff_dock)
+        self.addDockWidget(right, self.file_dock)
 
         # Update fields affected by model
         self.revtext.setText(dag.ref)
@@ -423,7 +434,7 @@ class DAGView(MainWindow):
         self.update_window_title()
 
         # Also re-loads dag.* from the saved state
-        if not qtutils.apply_state(self):
+        if not self.restore_state(settings=settings):
             self.resize_to_desktop()
 
         qtutils.connect_button(self.zoom_out, self.graphview.zoom_out)
@@ -469,23 +480,23 @@ class DAGView(MainWindow):
         qtutils.add_close_action(self)
 
     def text_changed(self, txt):
-        self.dag.ref = unicode(txt)
+        self.dag.ref = ustr(txt)
         self.update_window_title()
 
     def update_window_title(self):
         project = self.model.project
         if self.dag.ref:
-            self.setWindowTitle(N_('%s: %s - DAG') % (project, self.dag.ref))
+            self.setWindowTitle(N_('%(project)s: %(ref)s - DAG') % dict(project=project, ref=self.dag.ref))
         else:
             self.setWindowTitle(project + N_(' - DAG'))
 
     def export_state(self):
-        state = self.Mixin.export_state(self)
+        state = MainWindow.export_state(self)
         state['count'] = self.dag.count
         return state
 
     def apply_state(self, state):
-        result = self.Mixin.apply_state(self, state)
+        result = MainWindow.apply_state(self, state)
         try:
             count = state['count']
             if self.dag.overridden('count'):
@@ -510,7 +521,7 @@ class DAGView(MainWindow):
         self.display()
 
     def display(self):
-        new_ref = unicode(self.revtext.text())
+        new_ref = self.revtext.value()
         if not new_ref:
             return
         new_count = self.maxresults.value()
@@ -529,7 +540,7 @@ class DAGView(MainWindow):
         self.thread.start()
 
     def show(self):
-        self.Mixin.show(self)
+        MainWindow.show(self)
         self.treewidget.adjust_columns()
 
     def clear(self):
@@ -575,10 +586,10 @@ class DAGView(MainWindow):
     def closeEvent(self, event):
         self.revtext.close_popup()
         self.thread.stop()
-        self.Mixin.closeEvent(self, event)
+        MainWindow.closeEvent(self, event)
 
     def resizeEvent(self, e):
-        self.Mixin.resizeEvent(self, e)
+        MainWindow.resizeEvent(self, e)
         self.treewidget.adjust_columns()
 
 
@@ -1173,7 +1184,7 @@ class GraphView(ViewerMixin, QtGui.QGraphicsView):
         self_commits = self.commits
         self_items = self.items
 
-        commits = self_commits[-2:]
+        commits = self_commits[-8:]
         items = [self_items[c.sha1] for c in commits]
         self.fit_view_to_items(items)
 
@@ -1187,10 +1198,11 @@ class GraphView(ViewerMixin, QtGui.QGraphicsView):
         if not items:
             rect = self.scene().itemsBoundingRect()
         else:
-            x_min = sys.maxint
-            y_min = sys.maxint
-            x_max = -sys.maxint
-            ymax = -sys.maxint
+            maxint = 9223372036854775807
+            x_min = maxint
+            y_min = maxint
+            x_max = -maxint
+            ymax = -maxint
             for item in items:
                 pos = item.pos()
                 item_rect = item.boundingRect()
@@ -1224,10 +1236,10 @@ class GraphView(ViewerMixin, QtGui.QGraphicsView):
             item.setSelected(True)
 
     def handle_event(self, event_handler, event):
-        self.update()
         self.save_selection(event)
         event_handler(self, event)
         self.restore_selection(event)
+        self.update()
 
     def set_selecting(self, selecting):
         self.selecting = selecting
@@ -1301,9 +1313,9 @@ class GraphView(ViewerMixin, QtGui.QGraphicsView):
             max_ = scrollbar.maximum()
             range_ = max_ - min_
             distance = value - min_
-            nonzero_range = float(range_) > 0.1
+            nonzero_range = range_ > 0.1
             if nonzero_range:
-                scrolloffset = distance/float(range_)
+                scrolloffset = distance/range_
             else:
                 adjust_scrollbars = False
 
@@ -1374,7 +1386,7 @@ class GraphView(ViewerMixin, QtGui.QGraphicsView):
                 # shift them to the right to avoid overlapping edges
                 child_gens = [c.generation for c in node.children]
                 maxgen = max(child_gens)
-                for g in xrange(generation + 1, maxgen):
+                for g in range(generation + 1, maxgen):
                     x_offsets[g] += x_off
 
             if len(node.parents) == 1:
@@ -1416,7 +1428,7 @@ class GraphView(ViewerMixin, QtGui.QGraphicsView):
     def sort_by_generation(self, commits):
         if len(commits) < 2:
             return commits
-        commits.sort(cmp=lambda a, b: cmp(a.generation, b.generation))
+        commits.sort(key=lambda x: x.generation)
         return commits
 
     # Qt overrides
@@ -1445,6 +1457,8 @@ class GraphView(ViewerMixin, QtGui.QGraphicsView):
         self.last_mouse[0] = pos.x()
         self.last_mouse[1] = pos.y()
         self.handle_event(QtGui.QGraphicsView.mouseMoveEvent, event)
+        if self.pressed:
+            self.viewport().repaint()
 
     def mouseReleaseEvent(self, event):
         self.pressed = False
@@ -1453,6 +1467,7 @@ class GraphView(ViewerMixin, QtGui.QGraphicsView):
             return
         self.handle_event(QtGui.QGraphicsView.mouseReleaseEvent, event)
         self.selection_list = []
+        self.viewport().repaint()
 
     def wheelEvent(self, event):
         """Handle Qt mouse wheel events."""
