@@ -35,10 +35,23 @@ class DiffTextEdit(MonoTextView):
         self.highlighter = DiffSyntaxHighlighter(self.document(),
                                                  whitespace=whitespace)
 
+class DiffEditorWidget(QtGui.QWidget):
+
+    def __init__(self, parent=None):
+        QtGui.QWidget.__init__(self, parent)
+
+        self.editor = DiffEditor(self, parent.titleBarWidget())
+
+        self.main_layout = QtGui.QVBoxLayout()
+        self.main_layout.setSpacing(defs.spacing)
+        self.main_layout.setMargin(defs.no_margin)
+        self.main_layout.addWidget(self.editor)
+        self.setLayout(self.main_layout)
+
 
 class DiffEditor(DiffTextEdit):
 
-    def __init__(self, parent):
+    def __init__(self, parent, titlebar):
         DiffTextEdit.__init__(self, parent)
         self.model = model = main.model()
 
@@ -75,16 +88,15 @@ class DiffEditor(DiffTextEdit):
         self.diffopts_button.setMenu(self.diffopts_menu)
         qtutils.hide_button_menu_indicator(self.diffopts_button)
 
-        titlebar = parent.titleBarWidget()
         titlebar.add_corner_widget(self.diffopts_button)
 
-        self.action_process_section = qtutils.add_action(self,
-                N_('Process Section'),
-                self.apply_section, Qt.Key_H)
-        self.action_process_selection = qtutils.add_action(self,
-                N_('Process Selection'),
+        self.action_apply_selection = qtutils.add_action(self, '',
                 self.apply_selection, Qt.Key_S)
 
+        self.action_revert_selection = qtutils.add_action(self, '',
+                self.revert_selection)
+        self.action_revert_selection.setIcon(qtutils.icon('undo.svg'))
+
         self.launch_editor = qtutils.add_action(self,
                 cmds.LaunchEditor.name(), run(cmds.LaunchEditor),
                 cmds.LaunchEditor.SHORTCUT,
@@ -96,33 +108,8 @@ class DiffEditor(DiffTextEdit):
                 cmds.LaunchDifftool.SHORTCUT)
         self.launch_difftool.setIcon(qtutils.icon('git.svg'))
 
-        self.action_stage_selection = qtutils.add_action(self,
-                N_('Stage &Selected Lines'),
-                self.stage_selection)
-        self.action_stage_selection.setIcon(qtutils.icon('add.svg'))
-        self.action_stage_selection.setShortcut(Qt.Key_S)
-
-        self.action_revert_selection = qtutils.add_action(self,
-                N_('Revert Selected Lines...'),
-                self.revert_selection)
-        self.action_revert_selection.setIcon(qtutils.icon('undo.svg'))
-
-        self.action_unstage_selection = qtutils.add_action(self,
-                N_('Unstage &Selected Lines'),
-                self.unstage_selection)
-        self.action_unstage_selection.setIcon(qtutils.icon('remove.svg'))
-        self.action_unstage_selection.setShortcut(Qt.Key_S)
-
-        self.action_apply_selection = qtutils.add_action(self,
-                N_('Apply Diff Selection to Work Tree'),
-                self.stage_selection)
-        self.action_apply_selection.setIcon(qtutils.apply_icon())
-
         model.add_observer(model.message_diff_text_changed, self._emit_text)
 
-        self.connect(self, SIGNAL('copyAvailable(bool)'),
-                     self.enable_selection_actions)
-
         self.connect(self, SIGNAL('set_text'), self.setPlainText)
 
     def _emit_text(self, text):
@@ -147,8 +134,8 @@ class DiffEditor(DiffTextEdit):
         s = selection.selection()
         filename = selection.filename()
 
-        if self.model.stageable():
-            if s.modified and s.modified[0] in main.model().submodules:
+        if self.model.stageable() and s.modified:
+            if s.modified[0] in main.model().submodules:
                 action = menu.addAction(qtutils.icon('add.svg'),
                                         cmds.Stage.name(),
                                         cmds.run(cmds.Stage, s.modified))
@@ -157,16 +144,20 @@ class DiffEditor(DiffTextEdit):
                                N_('Launch git-cola'),
                                cmds.run(cmds.OpenRepo,
                                         core.abspath(s.modified[0])))
-            elif s.modified:
-                action = menu.addAction(qtutils.icon('add.svg'),
-                                        N_('Stage Section'),
-                                        self.stage_section)
-                action.setShortcut(Qt.Key_H)
-                menu.addAction(self.action_stage_selection)
-                menu.addSeparator()
-                menu.addAction(qtutils.icon('undo.svg'),
-                               N_('Revert Section...'),
-                               self.revert_section)
+            else:
+                if self.has_selection():
+                    apply_text = N_('Stage Selected Lines')
+                    revert_text = N_('Revert Selected Lines...')
+                else:
+                    apply_text = N_('Stage Diff Hunk')
+                    revert_text = N_('Revert Diff Hunk...')
+
+                self.action_apply_selection.setText(apply_text)
+                self.action_apply_selection.setIcon(qtutils.icon('add.svg'))
+
+                self.action_revert_selection.setText(revert_text)
+
+                menu.addAction(self.action_apply_selection)
                 menu.addAction(self.action_revert_selection)
 
         if self.model.unstageable():
@@ -180,11 +171,15 @@ class DiffEditor(DiffTextEdit):
                                cmds.do(cmds.OpenRepo,
                                        core.abspath(s.staged[0])))
             elif s.staged:
-                action = menu.addAction(qtutils.icon('remove.svg'),
-                                        N_('Unstage Section'),
-                                        self.unstage_section)
-                action.setShortcut(Qt.Key_H)
-                menu.addAction(self.action_unstage_selection)
+                if self.has_selection():
+                    apply_text = N_('Unstage Selected Lines')
+                else:
+                    apply_text = N_('Unstage Diff Hunk')
+
+                self.action_apply_selection.setText(apply_text)
+                self.action_apply_selection.setIcon(qtutils.icon('remove.svg'))
+
+                menu.addAction(self.action_apply_selection)
 
         if self.model.stageable() or self.model.unstageable():
             # Do not show the "edit" action when the file does not exist.
@@ -223,8 +218,7 @@ class DiffEditor(DiffTextEdit):
             # Intercept right-click to move the cursor to the current position.
             # setTextCursor() clears the selection so this is only done when
             # nothing is selected.
-            _, selection_text = self.offset_and_selection()
-            if not selection_text:
+            if not self.has_selection():
                 cursor = self.cursorForPosition(event.pos())
                 self.setTextCursor(cursor)
 
@@ -271,84 +265,50 @@ class DiffEditor(DiffTextEdit):
         if scrollbar and scrollvalue is not None:
             scrollbar.setValue(scrollvalue)
 
+    def has_selection(self):
+        cursor = self.textCursor()
+        return cursor.selectionStart() != cursor.selectionEnd()
+
     def offset_and_selection(self):
         cursor = self.textCursor()
-        offset = cursor.position()
+        offset = cursor.selectionStart()
         selection_text = ustr(cursor.selection().toPlainText())
         return offset, selection_text
 
-    # Mutators
-    def enable_selection_actions(self, enabled):
-        self.action_apply_selection.setEnabled(enabled)
-        self.action_revert_selection.setEnabled(enabled)
-        self.action_unstage_selection.setEnabled(enabled)
-        self.action_stage_selection.setEnabled(enabled)
-
-    def apply_section(self):
-        s = selection.single_selection()
-        if self.model.stageable() and s.modified:
-            self.stage_section()
-        elif self.model.unstageable():
-            self.unstage_section()
-
     def apply_selection(self):
         s = selection.single_selection()
         if self.model.stageable() and s.modified:
-            self.stage_selection()
+            self.process_diff_selection(staged=False)
         elif self.model.unstageable():
-            self.unstage_selection()
-
-    def stage_section(self):
-        """Stage a specific section."""
-        self.process_diff_selection(staged=False)
-
-    def stage_selection(self):
-        """Stage selected lines."""
-        self.process_diff_selection(staged=False, selected=True)
-
-    def unstage_section(self, cached=True):
-        """Unstage a section."""
-        self.process_diff_selection(staged=True)
-
-    def unstage_selection(self):
-        """Unstage selected lines."""
-        self.process_diff_selection(staged=True, selected=True)
-
-    def revert_section(self):
-        """Destructively remove a section from a worktree file."""
-        if not qtutils.confirm(N_('Revert Section?'),
-                               N_('This operation drops uncommitted changes.\n'
-                                  'These changes cannot be recovered.'),
-                               N_('Revert the uncommitted changes?'),
-                               N_('Revert Section'),
-                               default=True,
-                               icon=qtutils.icon('undo.svg')):
-            return
-        self.process_diff_selection(staged=False, apply_to_worktree=True,
-                                    reverse=True)
+            self.process_diff_selection(staged=True)
 
     def revert_selection(self):
-        """Destructively check out content for the selected file from $head."""
-        if not qtutils.confirm(N_('Revert Selected Lines?'),
+        """Destructively revert selected lines or hunk from a worktree file."""
+
+        if self.has_selection():
+            title = N_('Revert Selected Lines?')
+            ok_text = N_('Revert Selected Lines')
+        else:
+            title = N_('Revert Diff Hunk?')
+            ok_text = N_('Revert Diff Hunk')
+
+        if not qtutils.confirm(title,
                                N_('This operation drops uncommitted changes.\n'
                                   'These changes cannot be recovered.'),
                                N_('Revert the uncommitted changes?'),
-                               N_('Revert Selected Lines'),
+                               ok_text,
                                default=True,
                                icon=qtutils.icon('undo.svg')):
             return
-        self.process_diff_selection(staged=False, apply_to_worktree=True,
-                                    reverse=True, selected=True)
+        self.process_diff_selection(staged=False, apply_to_worktree=True)
 
-    def process_diff_selection(self, selected=False,
-                               staged=True, apply_to_worktree=False,
-                               reverse=False):
+    def process_diff_selection(self, staged=True, apply_to_worktree=False):
         """Implement un/staging of selected lines or sections."""
         if selection.selection_model().is_empty():
             return
         offset, selection_text = self.offset_and_selection()
         cmds.do(cmds.ApplyDiffSelection,
-                staged, selected, offset, selection_text, apply_to_worktree)
+                staged, offset, selection_text, apply_to_worktree)
 
 
 
@@ -363,7 +323,7 @@ class DiffWidget(QtGui.QWidget):
         summary_font = QtGui.QFont(author_font)
         summary_font.setWeight(QtGui.QFont.Bold)
 
-        policy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding,
+        policy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding,
                                    QtGui.QSizePolicy.Minimum)
 
         self.gravatar_label = gravatar.GravatarLabel()
