@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 from PyQt4 import QtCore
 from PyQt4 import QtGui
 from PyQt4.QtCore import Qt
@@ -13,6 +15,28 @@ from cola.qtutils import diff_font
 from cola.widgets import defs
 from cola.widgets.standard import Dialog
 from cola.widgets.text import HintedTextView, HintedLineEdit
+from cola.compat import ustr
+
+
+def grep():
+    """Prompt and use 'git grep' to find the content."""
+    widget = new_grep(parent=qtutils.active_window())
+    widget.show()
+    widget.raise_()
+    return widget
+
+
+def new_grep(text=None, parent=None):
+    widget = Grep(parent=parent)
+    if text:
+        widget.search_for(text)
+    return widget
+
+
+def goto_grep(line):
+    """Called when Search -> Grep's right-click 'goto' action."""
+    filename, line_number, contents = line.split(':', 2)
+    do(cmds.Edit, [filename], line_number=line_number)
 
 
 class GrepThread(QtCore.QThread):
@@ -40,11 +64,12 @@ class GrepThread(QtCore.QThread):
 
 class Grep(Dialog):
 
-    def __init__(self, parent):
+    def __init__(self, parent=None):
         Dialog.__init__(self, parent)
         self.setAttribute(Qt.WA_MacMetalStyle)
-        self.setWindowModality(Qt.WindowModal)
         self.setWindowTitle(N_('Search'))
+        if parent is not None:
+            self.setWindowModality(Qt.WindowModal)
 
         self.input_label = QtGui.QLabel('git grep')
         self.input_label.setFont(diff_font())
@@ -92,11 +117,11 @@ class Grep(Dialog):
         self.close_button = QtGui.QPushButton(N_('Close'))
 
         self.input_layout = QtGui.QHBoxLayout()
-        self.input_layout.setMargin(0)
+        self.input_layout.setMargin(defs.no_margin)
         self.input_layout.setSpacing(defs.button_spacing)
 
         self.bottom_layout = QtGui.QHBoxLayout()
-        self.bottom_layout.setMargin(0)
+        self.bottom_layout.setMargin(defs.no_margin)
         self.bottom_layout.setSpacing(defs.button_spacing)
 
         self.mainlayout = QtGui.QVBoxLayout()
@@ -137,24 +162,24 @@ class Grep(Dialog):
                            Qt.Key_Down, Qt.Key_Enter, Qt.Key_Return)
         qtutils.add_action(self, 'FocusSearch',
                            lambda: self.input_txt.setFocus(),
-                           'Ctrl+l')
+                           'Ctrl+L')
         qtutils.connect_button(self.edit_button, self.edit)
         qtutils.connect_button(self.refresh_button, self.search)
         qtutils.connect_toggle(self.shell_checkbox, lambda x: self.search())
         qtutils.connect_button(self.close_button, self.close)
         qtutils.add_close_action(self)
 
-        if not qtutils.apply_state(self):
+        if not self.restore_state():
             self.resize(666, 420)
 
     def done(self, exit_code):
-        qtutils.save_state(self)
+        self.save_state()
         return Dialog.done(self, exit_code)
 
     def regexp_mode(self):
         idx = self.regexp_combo.currentIndex()
         data = self.regexp_combo.itemData(idx, Qt.UserRole).toPyObject()
-        return unicode(data)
+        return ustr(data)
 
     def search(self):
         self.edit_button.setEnabled(False)
@@ -256,11 +281,11 @@ class GrepTextView(HintedTextView):
                 Qt.Key_W)
 
         qtutils.add_action(self, 'PageUp',
-                lambda: self.page(-self.height()/2),
+                lambda: self.page(-self.height()//2),
                 'Shift+Space')
 
         qtutils.add_action(self, 'PageDown',
-                lambda: self.page(self.height()/2),
+                lambda: self.page(self.height()//2),
                 Qt.Key_Space)
 
     def contextMenuEvent(self, event):
@@ -310,16 +335,3 @@ class GrepTextView(HintedTextView):
                 # change focus.
                 self.emit(SIGNAL('leave()'))
         return HintedTextView.keyPressEvent(self, event)
-
-
-def goto_grep(line):
-    """Called when Search -> Grep's right-click 'goto' action."""
-    filename, line_number, contents = line.split(':', 2)
-    do(cmds.Edit, [filename], line_number=line_number)
-
-
-def run_grep(text=None, parent=None):
-    widget = Grep(parent)
-    if text:
-        widget.search_for(text)
-    return widget
