@@ -5,6 +5,7 @@ import time
 from PyQt4 import QtGui
 
 from cola.i18n import N_
+from cola.widgets import defs
 from cola.widgets.text import MonoTextView
 
 
@@ -14,7 +15,7 @@ class LogWidget(QtGui.QWidget):
         QtGui.QWidget.__init__(self, parent)
 
         self._layout = QtGui.QVBoxLayout(self)
-        self._layout.setMargin(0)
+        self._layout.setMargin(defs.no_margin)
 
         self.output_text = MonoTextView(self)
         self._layout.addWidget(self.output_text)
@@ -28,13 +29,14 @@ class LogWidget(QtGui.QWidget):
         self.output_text.setText(output)
 
     def log_status(self, status, out, err=None):
-        msg = out
+        msg = []
+        if out:
+            msg.append(out)
         if err:
-            msg += '\n' + err
-        if status != 0:
-            msg += '\n'
-            msg += N_('exit code %s') % status
-        self.log(msg)
+            msg.append(err)
+        if status:
+            msg.append(N_('exit code %s') % status)
+        self.log('\n'.join(msg))
 
     def log(self, msg):
         if not msg:
