@@ -1,5 +1,7 @@
 """This view provides the main git-cola user interface.
 """
+from __future__ import division, absolute_import, unicode_literals
+
 import os
 
 from PyQt4 import QtCore
@@ -14,12 +16,12 @@ from cola import guicmds
 from cola import gitcfg
 from cola import qtutils
 from cola import resources
-from cola import settings
 from cola import utils
 from cola import version
-from cola.bookmarks import manage_bookmarks
+from cola.compat import unichr
 from cola.git import git
 from cola.git import STDOUT
+from cola.guicmds import TaskRunner
 from cola.i18n import N_
 from cola.interaction import Interaction
 from cola.models import prefs
@@ -29,11 +31,12 @@ from cola.qtutils import connect_action
 from cola.qtutils import connect_action_bool
 from cola.qtutils import create_dock
 from cola.qtutils import create_menu
-from cola.qtutils import create_toolbutton
-from cola.qtutils import options_icon
+from cola.settings import Settings
 from cola.widgets import action
+from cola.widgets import bookmarks
 from cola.widgets import cfgactions
 from cola.widgets import editremotes
+from cola.widgets import merge
 from cola.widgets import remote
 from cola.widgets.about import launch_about_dialog
 from cola.widgets.about import show_shortcuts
@@ -45,11 +48,13 @@ from cola.widgets.compare import compare_branches
 from cola.widgets.createtag import create_tag
 from cola.widgets.createbranch import create_new_branch
 from cola.widgets.dag import git_dag
-from cola.widgets.diff import DiffEditor
+from cola.widgets.diff import DiffEditorWidget
+from cola.widgets.grep import grep
 from cola.widgets.log import LogWidget
-from cola.widgets import merge
+from cola.widgets.patch import apply_patches
 from cola.widgets.prefs import preferences
-from cola.widgets.recent import browse_recent
+from cola.widgets.recent import browse_recent_files
+from cola.widgets.standard import ProgressDialog
 from cola.widgets.status import StatusWidget
 from cola.widgets.search import search
 from cola.widgets.standard import MainWindow
@@ -58,11 +63,13 @@ from cola.widgets.stash import stash
 
 class MainView(MainWindow):
 
-    def __init__(self, model, parent=None):
+    def __init__(self, model, parent=None, settings=None):
         MainWindow.__init__(self, parent)
+        self.setAttribute(Qt.WA_MacMetalStyle)
+
         # Default size; this is thrown out when save/restore is used
-        self.resize(987, 610)
         self.model = model
+        self.settings = settings
         self.prefs_model = prefs_model = prefs.PreferencesModel()
 
         # The widget version is used by import/export_state().
@@ -72,8 +79,9 @@ class MainView(MainWindow):
         # Keeps track of merge messages we've seen
         self.merge_message_hash = ''
 
-        self.setAcceptDrops(True)
-        self.setAttribute(Qt.WA_MacMetalStyle)
+        # Runs asynchronous tasks
+        self.task_runner = TaskRunner(self)
+        self.progress = ProgressDialog('', '', self)
 
         cfg = gitcfg.instance()
         self.browser_dockable = (cfg.get('cola.browserdockable') or
@@ -91,15 +99,35 @@ class MainView(MainWindow):
         self.actionsdockwidget.hide()
 
         # "Repository Status" widget
-        self.statuswidget = StatusWidget(self)
         self.statusdockwidget = create_dock(N_('Status'), self)
+        self.statuswidget = StatusWidget(self.statusdockwidget.titleBarWidget(),
+                                         parent=self.statusdockwidget)
         self.statusdockwidget.setWidget(self.statuswidget)
 
+        # "Switch Repository" widgets
+        self.bookmarksdockwidget = create_dock(N_('Bookmarks'), self)
+        self.bookmarkswidget = bookmarks.BookmarksWidget(
+                bookmarks.BOOKMARKS, parent=self.bookmarksdockwidget)
+        self.bookmarksdockwidget.setWidget(self.bookmarkswidget)
+
+        self.recentdockwidget = create_dock(N_('Recent'), self)
+        self.recentwidget = bookmarks.BookmarksWidget(
+                bookmarks.RECENT_REPOS, parent=self.recentdockwidget)
+        self.recentdockwidget.setWidget(self.recentwidget)
+        self.recentdockwidget.hide()
+
         # "Commit Message Editor" widget
         self.position_label = QtGui.QLabel()
         font = qtutils.default_monospace_font()
         font.setPointSize(int(font.pointSize() * 0.8))
         self.position_label.setFont(font)
+
+        # make the position label fixed size to avoid layout issues
+        fm = self.position_label.fontMetrics()
+        width = fm.width('999:999')
+        height = self.position_label.sizeHint().height()
+        self.position_label.setFixedSize(width, height)
+
         self.commitdockwidget = create_dock(N_('Commit'), self)
         titlebar = self.commitdockwidget.titleBarWidget()
         titlebar.add_corner_widget(self.position_label)
@@ -116,187 +144,155 @@ class MainView(MainWindow):
 
         # "Diff Viewer" widget
         self.diffdockwidget = create_dock(N_('Diff'), self)
-        self.diffeditor = DiffEditor(self.diffdockwidget)
-        self.diffdockwidget.setWidget(self.diffeditor)
-
-        # "Diff Options" tool menu
-        self.diff_ignore_space_at_eol_action = add_action(self,
-                N_('Ignore changes in whitespace at EOL'),
-                self._update_diff_opts)
-        self.diff_ignore_space_at_eol_action.setCheckable(True)
-
-        self.diff_ignore_space_change_action = add_action(self,
-                N_('Ignore changes in amount of whitespace'),
-                self._update_diff_opts)
-        self.diff_ignore_space_change_action.setCheckable(True)
-
-        self.diff_ignore_all_space_action = add_action(self,
-                N_('Ignore all whitespace'),
-                self._update_diff_opts)
-        self.diff_ignore_all_space_action.setCheckable(True)
-
-        self.diff_function_context_action = add_action(self,
-                N_('Show whole surrounding functions of changes'),
-                self._update_diff_opts)
-        self.diff_function_context_action.setCheckable(True)
-
-        self.diffopts_button = create_toolbutton(text=N_('Options'),
-                                                 icon=options_icon(),
-                                                 tooltip=N_('Diff Options'))
-        self.diffopts_menu = create_menu(N_('Diff Options'),
-                                         self.diffopts_button)
-
-        self.diffopts_menu.addAction(self.diff_ignore_space_at_eol_action)
-        self.diffopts_menu.addAction(self.diff_ignore_space_change_action)
-        self.diffopts_menu.addAction(self.diff_ignore_all_space_action)
-        self.diffopts_menu.addAction(self.diff_function_context_action)
-        self.diffopts_button.setMenu(self.diffopts_menu)
-        self.diffopts_button.setPopupMode(QtGui.QToolButton.InstantPopup)
-
-        titlebar = self.diffdockwidget.titleBarWidget()
-        titlebar.add_corner_widget(self.diffopts_button)
+        self.diffeditorwidget = DiffEditorWidget(self.diffdockwidget)
+        self.diffeditor = self.diffeditorwidget.editor
+        self.diffdockwidget.setWidget(self.diffeditorwidget)
 
         # All Actions
-        self.menu_unstage_all = add_action(self,
+        self.unstage_all_action = add_action(self,
                 N_('Unstage All'), cmds.run(cmds.UnstageAll))
-        self.menu_unstage_all.setIcon(qtutils.icon('remove.svg'))
+        self.unstage_all_action.setIcon(qtutils.icon('remove.svg'))
 
-        self.menu_unstage_selected = add_action(self,
+        self.unstage_selected_action = add_action(self,
                 N_('Unstage From Commit'), cmds.run(cmds.UnstageSelected))
-        self.menu_unstage_selected.setIcon(qtutils.icon('remove.svg'))
+        self.unstage_selected_action.setIcon(qtutils.icon('remove.svg'))
 
-        self.menu_show_diffstat = add_action(self,
+        self.show_diffstat_action = add_action(self,
                 N_('Diffstat'), cmds.run(cmds.Diffstat), 'Alt+D')
 
-        self.menu_stage_modified = add_action(self,
+        self.stage_modified_action = add_action(self,
                 N_('Stage Changed Files To Commit'),
                 cmds.run(cmds.StageModified), 'Alt+A')
-        self.menu_stage_modified.setIcon(qtutils.icon('add.svg'))
+        self.stage_modified_action.setIcon(qtutils.icon('add.svg'))
 
-        self.menu_stage_untracked = add_action(self,
+        self.stage_untracked_action = add_action(self,
                 N_('Stage All Untracked'),
                 cmds.run(cmds.StageUntracked), 'Alt+U')
-        self.menu_stage_untracked.setIcon(qtutils.icon('add.svg'))
+        self.stage_untracked_action.setIcon(qtutils.icon('add.svg'))
 
-        self.menu_export_patches = add_action(self,
+        self.apply_patches_action = add_action(self,
+                N_('Apply Patches...'), apply_patches)
+
+        self.export_patches_action = add_action(self,
                 N_('Export Patches...'), guicmds.export_patches, 'Alt+E')
 
-        self.new_repository = add_action(self,
+        self.new_repository_action = add_action(self,
                 N_('New Repository...'), guicmds.open_new_repo)
-        self.new_repository.setIcon(qtutils.new_icon())
+        self.new_repository_action.setIcon(qtutils.new_icon())
 
-        self.menu_preferences = add_action(self,
+        self.preferences_action = add_action(self,
                 N_('Preferences'), self.preferences,
                 QtGui.QKeySequence.Preferences, 'Ctrl+O')
 
-        self.menu_edit_remotes = add_action(self,
-                N_('Edit Remotes...'), lambda: editremotes.edit().exec_())
-        self.menu_rescan = add_action(self,
+        self.edit_remotes_action = add_action(self,
+                N_('Edit Remotes...'), lambda: editremotes.remote_editor().exec_())
+        self.rescan_action = add_action(self,
                 cmds.Refresh.name(),
                 cmds.run(cmds.Refresh),
                 cmds.Refresh.SHORTCUT)
-        self.menu_rescan.setIcon(qtutils.reload_icon())
+        self.rescan_action.setIcon(qtutils.reload_icon())
 
-        self.menu_browse_recent = add_action(self,
+        self.browse_recently_modified_action = add_action(self,
                 N_('Recently Modified Files...'),
-                browse_recent, 'Shift+Ctrl+E')
+                browse_recent_files, 'Shift+Ctrl+E')
 
-        self.menu_cherry_pick = add_action(self,
+        self.cherry_pick_action = add_action(self,
                 N_('Cherry-Pick...'),
-                guicmds.cherry_pick, 'Ctrl+P')
+                guicmds.cherry_pick, 'Shift+Ctrl+C')
 
-        self.menu_load_commitmsg = add_action(self,
+        self.load_commitmsg_action = add_action(self,
                 N_('Load Commit Message...'), guicmds.load_commitmsg)
 
-        self.menu_save_tarball = add_action(self,
+        self.save_tarball_action = add_action(self,
                 N_('Save As Tarball/Zip...'), self.save_archive)
 
-        self.menu_quit = add_action(self,
+        self.quit_action = add_action(self,
                 N_('Quit'), self.close, 'Ctrl+Q')
-        self.menu_manage_bookmarks = add_action(self,
-                N_('Bookmarks...'), manage_bookmarks)
-        self.menu_grep = add_action(self,
-                N_('Grep'), guicmds.grep, 'Ctrl+G')
-        self.menu_merge_local = add_action(self,
+        self.grep_action = add_action(self,
+                N_('Grep'), grep, 'Ctrl+G')
+        self.merge_local_action = add_action(self,
                 N_('Merge...'), merge.local_merge)
 
-        self.menu_merge_abort = add_action(self,
+        self.merge_abort_action = add_action(self,
                 N_('Abort Merge...'), merge.abort_merge)
 
-        self.menu_fetch = add_action(self,
-                N_('Fetch...'), remote.fetch)
-        self.menu_push = add_action(self,
-                N_('Push...'), remote.push)
-        self.menu_pull = add_action(self,
-                N_('Pull...'), remote.pull)
+        self.fetch_action = add_action(self,
+                N_('Fetch...'), remote.fetch, 'Ctrl+F')
+        self.push_action = add_action(self,
+                N_('Push...'), remote.push, 'Ctrl+P')
+        self.pull_action = add_action(self,
+                N_('Pull...'), remote.pull, 'Shift+Ctrl+P')
 
-        self.menu_open_repo = add_action(self,
+        self.open_repo_action = add_action(self,
                 N_('Open...'), guicmds.open_repo)
-        self.menu_open_repo.setIcon(qtutils.open_icon())
+        self.open_repo_action.setIcon(qtutils.open_icon())
 
-        self.menu_stash = add_action(self,
+        self.open_repo_new_action = add_action(self,
+                N_('Open in New Window...'), guicmds.open_repo_in_new_window)
+        self.open_repo_new_action.setIcon(qtutils.open_icon())
+
+        self.stash_action = add_action(self,
                 N_('Stash...'), stash, 'Alt+Shift+S')
 
-        self.menu_clone_repo = add_action(self,
-                N_('Clone...'), guicmds.clone_repo)
-        self.menu_clone_repo.setIcon(qtutils.git_icon())
+        self.clone_repo_action = add_action(self,
+                N_('Clone...'), self.clone_repo)
+        self.clone_repo_action.setIcon(qtutils.git_icon())
 
-        self.menu_help_docs = add_action(self,
+        self.help_docs_action = add_action(self,
                 N_('Documentation'), resources.show_html_docs,
                 QtGui.QKeySequence.HelpContents)
 
-        self.menu_help_shortcuts = add_action(self,
+        self.help_shortcuts_action = add_action(self,
                 N_('Keyboard Shortcuts'),
                 show_shortcuts,
                 QtCore.Qt.Key_Question)
 
-        self.menu_visualize_current = add_action(self,
+        self.visualize_current_action = add_action(self,
                 N_('Visualize Current Branch...'),
                 cmds.run(cmds.VisualizeCurrent))
-        self.menu_visualize_all = add_action(self,
+        self.visualize_all_action = add_action(self,
                 N_('Visualize All Branches...'),
                 cmds.run(cmds.VisualizeAll))
-        self.menu_search_commits = add_action(self,
+        self.search_commits_action = add_action(self,
                 N_('Search...'), search)
-        self.menu_browse_branch = add_action(self,
+        self.browse_branch_action = add_action(self,
                 N_('Browse Current Branch...'), guicmds.browse_current)
-        self.menu_browse_other_branch = add_action(self,
+        self.browse_other_branch_action = add_action(self,
                 N_('Browse Other Branch...'), guicmds.browse_other)
-        self.menu_load_commitmsg_template = add_action(self,
+        self.load_commitmsg_template_action = add_action(self,
                 N_('Get Commit Message Template'),
                 cmds.run(cmds.LoadCommitMessageFromTemplate))
-        self.menu_help_about = add_action(self,
+        self.help_about_action = add_action(self,
                 N_('About'), launch_about_dialog)
 
-        self.menu_diff_expression = add_action(self,
+        self.diff_expression_action = add_action(self,
                 N_('Expression...'), guicmds.diff_expression)
-        self.menu_branch_compare = add_action(self,
+        self.branch_compare_action = add_action(self,
                 N_('Branches...'), compare_branches)
 
-        self.menu_create_tag = add_action(self,
+        self.create_tag_action = add_action(self,
                 N_('Create Tag...'), create_tag)
 
-        self.menu_create_branch = add_action(self,
+        self.create_branch_action = add_action(self,
                 N_('Create...'), create_new_branch, 'Ctrl+B')
 
-        self.menu_delete_branch = add_action(self,
+        self.delete_branch_action = add_action(self,
                 N_('Delete...'), guicmds.delete_branch)
 
-        self.menu_delete_remote_branch = add_action(self,
+        self.delete_remote_branch_action = add_action(self,
                 N_('Delete Remote Branch...'), guicmds.delete_remote_branch)
 
-        self.menu_checkout_branch = add_action(self,
+        self.checkout_branch_action = add_action(self,
                 N_('Checkout...'), guicmds.checkout_branch, 'Alt+B')
-        self.menu_branch_review = add_action(self,
+        self.branch_review_action = add_action(self,
                 N_('Review...'), guicmds.review_branch)
 
-        self.menu_browse = add_action(self,
-                N_('Browser...'), worktree_browser)
-        self.menu_browse.setIcon(qtutils.git_icon())
+        self.browse_action = add_action(self,
+                N_('File Browser...'), worktree_browser)
+        self.browse_action.setIcon(qtutils.git_icon())
 
-        self.menu_dag = add_action(self,
-                N_('DAG...'), lambda: git_dag(self.model).show())
-        self.menu_dag.setIcon(qtutils.git_icon())
+        self.dag_action = add_action(self, N_('DAG...'), self.git_dag)
+        self.dag_action.setIcon(qtutils.git_icon())
 
         self.rebase_start_action = add_action(self,
                 N_('Start Interactive Rebase...'), self.rebase_start)
@@ -314,14 +310,17 @@ class MainView(MainWindow):
                 N_('Abort'), self.rebase_abort)
 
         # Relayed actions
+        status_tree = self.statusdockwidget.widget().tree
+        self.addAction(status_tree.revert_unstaged_edits_action)
+        self.addAction(status_tree.delete_untracked_files_action)
+
         if not self.browser_dockable:
             # These shortcuts conflict with those from the
             # 'Browser' widget so don't register them when
             # the browser is a dockable tool.
-            status_tree = self.statusdockwidget.widget().tree
-            self.addAction(status_tree.up)
-            self.addAction(status_tree.down)
-            self.addAction(status_tree.process_selection)
+            self.addAction(status_tree.up_action)
+            self.addAction(status_tree.down_action)
+            self.addAction(status_tree.process_selection_action)
 
         self.lock_layout_action = add_action_bool(self,
                 N_('Lock Layout'), self.set_lock_layout, False)
@@ -331,76 +330,76 @@ class MainView(MainWindow):
 
         # File Menu
         self.file_menu = create_menu(N_('File'), self.menubar)
-        self.file_menu.addAction(self.new_repository)
-        self.file_menu.addAction(self.menu_open_repo)
-        self.menu_open_recent = self.file_menu.addMenu(N_('Open Recent'))
+        self.open_recent_menu = self.file_menu.addMenu(N_('Open Recent'))
+        self.open_recent_menu.setIcon(qtutils.open_icon())
+        self.file_menu.addAction(self.open_repo_action)
+        self.file_menu.addAction(self.open_repo_new_action)
+        self.file_menu.addAction(self.clone_repo_action)
+        self.file_menu.addAction(self.new_repository_action)
         self.file_menu.addSeparator()
-        self.file_menu.addAction(self.menu_clone_repo)
-        self.file_menu.addAction(self.menu_manage_bookmarks)
+        self.file_menu.addAction(self.rescan_action)
+        self.file_menu.addAction(self.edit_remotes_action)
+        self.file_menu.addAction(self.browse_recently_modified_action)
         self.file_menu.addSeparator()
-        self.file_menu.addAction(self.menu_edit_remotes)
-        self.file_menu.addAction(self.menu_rescan)
+        self.file_menu.addAction(self.load_commitmsg_action)
+        self.file_menu.addAction(self.load_commitmsg_template_action)
         self.file_menu.addSeparator()
-        self.file_menu.addAction(self.menu_browse_recent)
+        self.file_menu.addAction(self.apply_patches_action)
+        self.file_menu.addAction(self.export_patches_action)
+        self.file_menu.addAction(self.save_tarball_action)
         self.file_menu.addSeparator()
-        self.file_menu.addAction(self.menu_load_commitmsg)
-        self.file_menu.addAction(self.menu_load_commitmsg_template)
-        self.file_menu.addSeparator()
-        self.file_menu.addAction(self.menu_save_tarball)
-        self.file_menu.addAction(self.menu_export_patches)
-        self.file_menu.addSeparator()
-        self.file_menu.addAction(self.menu_preferences)
-        self.file_menu.addAction(self.menu_quit)
+        self.file_menu.addAction(self.preferences_action)
+        self.file_menu.addAction(self.quit_action)
         self.menubar.addAction(self.file_menu.menuAction())
 
         # Actions menu
         self.actions_menu = create_menu(N_('Actions'), self.menubar)
-        self.actions_menu.addAction(self.menu_fetch)
-        self.actions_menu.addAction(self.menu_push)
-        self.actions_menu.addAction(self.menu_pull)
-        self.actions_menu.addAction(self.menu_stash)
+        self.actions_menu.addAction(self.fetch_action)
+        self.actions_menu.addAction(self.push_action)
+        self.actions_menu.addAction(self.pull_action)
+        self.actions_menu.addAction(self.stash_action)
         self.actions_menu.addSeparator()
-        self.actions_menu.addAction(self.menu_create_tag)
-        self.actions_menu.addAction(self.menu_cherry_pick)
-        self.actions_menu.addAction(self.menu_merge_local)
-        self.actions_menu.addAction(self.menu_merge_abort)
+        self.actions_menu.addAction(self.create_tag_action)
+        self.actions_menu.addAction(self.cherry_pick_action)
+        self.actions_menu.addAction(self.merge_local_action)
+        self.actions_menu.addAction(self.merge_abort_action)
         self.actions_menu.addSeparator()
-        self.actions_menu.addAction(self.menu_grep)
-        self.actions_menu.addAction(self.menu_search_commits)
+        self.actions_menu.addAction(self.grep_action)
+        self.actions_menu.addAction(self.search_commits_action)
         self.menubar.addAction(self.actions_menu.menuAction())
 
-        # Index Menu
-        self.commit_menu = create_menu(N_('Index'), self.menubar)
-        self.commit_menu.setTitle(N_('Index'))
-        self.commit_menu.addAction(self.menu_stage_modified)
-        self.commit_menu.addAction(self.menu_stage_untracked)
+        # Staging Area Menu
+        self.commit_menu = create_menu(N_('Staging Area'), self.menubar)
+        self.commit_menu.setTitle(N_('Staging Area'))
+        self.commit_menu.addAction(self.stage_modified_action)
+        self.commit_menu.addAction(self.stage_untracked_action)
         self.commit_menu.addSeparator()
-        self.commit_menu.addAction(self.menu_unstage_all)
-        self.commit_menu.addAction(self.menu_unstage_selected)
+        self.commit_menu.addAction(self.unstage_all_action)
+        self.commit_menu.addAction(self.unstage_selected_action)
         self.menubar.addAction(self.commit_menu.menuAction())
 
         # Diff Menu
         self.diff_menu = create_menu(N_('Diff'), self.menubar)
-        self.diff_menu.addAction(self.menu_diff_expression)
-        self.diff_menu.addAction(self.menu_branch_compare)
+        self.diff_menu.addAction(self.diff_expression_action)
+        self.diff_menu.addAction(self.branch_compare_action)
         self.diff_menu.addSeparator()
-        self.diff_menu.addAction(self.menu_show_diffstat)
+        self.diff_menu.addAction(self.show_diffstat_action)
         self.menubar.addAction(self.diff_menu.menuAction())
 
         # Branch Menu
         self.branch_menu = create_menu(N_('Branch'), self.menubar)
-        self.branch_menu.addAction(self.menu_branch_review)
+        self.branch_menu.addAction(self.branch_review_action)
         self.branch_menu.addSeparator()
-        self.branch_menu.addAction(self.menu_create_branch)
-        self.branch_menu.addAction(self.menu_checkout_branch)
-        self.branch_menu.addAction(self.menu_delete_branch)
-        self.branch_menu.addAction(self.menu_delete_remote_branch)
+        self.branch_menu.addAction(self.create_branch_action)
+        self.branch_menu.addAction(self.checkout_branch_action)
+        self.branch_menu.addAction(self.delete_branch_action)
+        self.branch_menu.addAction(self.delete_remote_branch_action)
         self.branch_menu.addSeparator()
-        self.branch_menu.addAction(self.menu_browse_branch)
-        self.branch_menu.addAction(self.menu_browse_other_branch)
+        self.branch_menu.addAction(self.browse_branch_action)
+        self.branch_menu.addAction(self.browse_other_branch_action)
         self.branch_menu.addSeparator()
-        self.branch_menu.addAction(self.menu_visualize_current)
-        self.branch_menu.addAction(self.menu_visualize_all)
+        self.branch_menu.addAction(self.visualize_current_action)
+        self.branch_menu.addAction(self.visualize_all_action)
         self.menubar.addAction(self.branch_menu.menuAction())
 
         # Rebase menu
@@ -416,8 +415,8 @@ class MainView(MainWindow):
 
         # View Menu
         self.view_menu = create_menu(N_('View'), self.menubar)
-        self.view_menu.addAction(self.menu_browse)
-        self.view_menu.addAction(self.menu_dag)
+        self.view_menu.addAction(self.browse_action)
+        self.view_menu.addAction(self.dag_action)
         self.view_menu.addSeparator()
         if self.browser_dockable:
             self.view_menu.addAction(self.browserdockwidget.toggleViewAction())
@@ -429,9 +428,9 @@ class MainView(MainWindow):
 
         # Help Menu
         self.help_menu = create_menu(N_('Help'), self.menubar)
-        self.help_menu.addAction(self.menu_help_docs)
-        self.help_menu.addAction(self.menu_help_shortcuts)
-        self.help_menu.addAction(self.menu_help_about)
+        self.help_menu.addAction(self.help_docs_action)
+        self.help_menu.addAction(self.help_shortcuts_action)
+        self.help_menu.addAction(self.help_about_action)
         self.menubar.addAction(self.help_menu.menuAction())
 
         # Set main menu
@@ -447,11 +446,13 @@ class MainView(MainWindow):
             self.addDockWidget(left, self.browserdockwidget)
             self.tabifyDockWidget(self.browserdockwidget, self.commitdockwidget)
         self.addDockWidget(left, self.diffdockwidget)
+        self.addDockWidget(right, self.statusdockwidget)
+        self.addDockWidget(right, self.bookmarksdockwidget)
+        self.addDockWidget(right, self.recentdockwidget)
         self.addDockWidget(bottom, self.actionsdockwidget)
         self.addDockWidget(bottom, self.logdockwidget)
         self.tabifyDockWidget(self.actionsdockwidget, self.logdockwidget)
 
-        self.addDockWidget(right, self.statusdockwidget)
 
         # Listen for model notifications
         model.add_observer(model.message_updated, self._update)
@@ -463,11 +464,15 @@ class MainView(MainWindow):
         # Set a default value
         self.show_cursor_position(1, 0)
 
-        self.connect(self.menu_open_recent, SIGNAL('aboutToShow()'),
+        self.connect(self.open_recent_menu, SIGNAL('aboutToShow()'),
                      self.build_recent_menu)
 
         self.connect(self.commitmsgeditor, SIGNAL('cursorPosition(int,int)'),
                      self.show_cursor_position)
+
+        self.connect(self.diffeditor, SIGNAL('diff_options_updated()'),
+                     self.statuswidget.refresh)
+
         self.connect(self, SIGNAL('update'), self._update_callback)
         self.connect(self, SIGNAL('install_config_actions'),
                      self._install_config_actions)
@@ -477,7 +482,8 @@ class MainView(MainWindow):
         self.install_config_actions()
 
         # Restore saved settings
-        if not qtutils.apply_state(self):
+        if not self.restore_state(settings=settings):
+            self.resize(987, 610)
             self.set_initial_size()
 
         self.statusdockwidget.widget().setFocus()
@@ -492,6 +498,9 @@ class MainView(MainWindow):
         self.statuswidget.set_initial_size()
         self.commitmsgeditor.set_initial_size()
 
+    def set_filter(self, txt):
+        self.statuswidget.set_filter(txt)
+
     # Qt overrides
     def closeEvent(self, event):
         """Save state in the settings manager."""
@@ -500,14 +509,17 @@ class MainView(MainWindow):
         MainWindow.closeEvent(self, event)
 
     def build_recent_menu(self):
-        recent = settings.Settings().recent
-        menu = self.menu_open_recent
+        settings = Settings()
+        settings.load()
+        recent = settings.recent
+        cmd = cmds.OpenRepo
+        menu = self.open_recent_menu
         menu.clear()
         for r in recent:
             name = os.path.basename(r)
             directory = os.path.dirname(r)
             text = '%s %s %s' % (name, unichr(0x2192), directory)
-            menu.addAction(text, cmds.run(cmds.OpenRepo, r))
+            menu.addAction(text, cmds.run(cmd, r))
 
     # Accessors
     mode = property(lambda self: self.model.mode)
@@ -626,10 +638,19 @@ class MainView(MainWindow):
         self.rebase_skip_action.setEnabled(is_rebasing)
         self.rebase_abort_action.setEnabled(is_rebasing)
 
+    def export_state(self):
+        state = MainWindow.export_state(self)
+        show_status_filter = self.statuswidget.filter_widget.isVisible()
+        state['show_status_filter'] = show_status_filter
+        return state
+
     def apply_state(self, state):
         """Imports data for save/restore"""
         result = MainWindow.apply_state(self, state)
         self.lock_layout_action.setChecked(state.get('lock_layout', False))
+
+        show_status_filter = state.get('show_status_filter', False)
+        self.statuswidget.filter_widget.setVisible(show_status_filter)
         return result
 
     def setup_dockwidget_view_menu(self):
@@ -644,6 +665,8 @@ class MainView(MainWindow):
             (optkey + '+2', self.statusdockwidget),
             (optkey + '+3', self.diffdockwidget),
             (optkey + '+4', self.actionsdockwidget),
+            (optkey + '+5', self.bookmarksdockwidget),
+            (optkey + '+6', self.recentdockwidget),
         )
         for shortcut, dockwidget in dockwidgets:
             # Associate the action with the shortcut
@@ -670,52 +693,18 @@ class MainView(MainWindow):
             self.addAction(toggleview)
             connect_action(toggleview, focusdock)
 
-    def _update_diff_opts(self):
-        space_at_eol = self.diff_ignore_space_at_eol_action.isChecked()
-        space_change = self.diff_ignore_space_change_action.isChecked()
-        all_space = self.diff_ignore_all_space_action.isChecked()
-        function_context = self.diff_function_context_action.isChecked()
-
-        gitcmds.update_diff_overrides(space_at_eol,
-                                      space_change,
-                                      all_space,
-                                      function_context)
-        self.statuswidget.refresh()
-
     def preferences(self):
         return preferences(model=self.prefs_model, parent=self)
 
+    def git_dag(self):
+        view = git_dag(self.model)
+        view.show()
+        view.raise_()
+
     def save_archive(self):
         ref = git.rev_parse('HEAD')[STDOUT]
         shortref = ref[:7]
-        GitArchiveDialog.save(ref, shortref, self)
-
-    def dragEnterEvent(self, event):
-        """Accepts drops"""
-        MainWindow.dragEnterEvent(self, event)
-        event.acceptProposedAction()
-
-    def dropEvent(self, event):
-        """Apply dropped patches with git-am"""
-        event.accept()
-        urls = event.mimeData().urls()
-        if not urls:
-            return
-        paths = map(lambda x: unicode(x.path()), urls)
-        patches = [p for p in paths if p.endswith('.patch')]
-        dirs = [p for p in paths if os.path.isdir(p)]
-        dirs.sort()
-        for d in dirs:
-            patches.extend(self._gather_patches(d))
-        cmds.do(cmds.ApplyPatches, patches)
-
-    def _gather_patches(self, path):
-        """Find patches in a subdirectory"""
-        patches = []
-        for root, subdirs, files in os.walk(path):
-            for name in [f for f in files if f.endswith('.patch')]:
-                patches.append(os.path.join(root, name))
-        return patches
+        GitArchiveDialog.save_hashed_objects(ref, shortref, self)
 
     def show_cursor_position(self, rows, cols):
         display = '&nbsp;%02d:%02d&nbsp;' % (rows, cols)
@@ -737,6 +726,11 @@ class MainView(MainWindow):
         self.position_label.setText(display)
 
     def rebase_start(self):
+        if self.model.staged or self.model.unmerged or self.model.modified:
+            Interaction.information(
+                    N_('Unable to rebase'),
+                    N_('You cannot rebase with uncommitted changes.'))
+            return
         branch = guicmds.choose_ref(N_('Select New Upstream'),
                                     N_('Interactive Rebase'))
         if not branch:
@@ -756,3 +750,7 @@ class MainView(MainWindow):
 
     def rebase_abort(self):
         cmds.do(cmds.RebaseAbort)
+
+    def clone_repo(self):
+        guicmds.clone_repo(self.task_runner, self.progress,
+                           guicmds.report_clone_repo_errors, True)
