@@ -1,3 +1,5 @@
+from __future__ import division, absolute_import, unicode_literals
+
 from PyQt4 import QtGui
 from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
@@ -9,6 +11,7 @@ from cola.i18n import N_
 from cola.models import main
 from cola.widgets import completion
 from cola.widgets import defs
+from cola.compat import ustr
 
 
 def local_merge():
@@ -40,8 +43,9 @@ class MergeView(QtGui.QDialog):
     def __init__(self, model, parent=None):
         QtGui.QDialog.__init__(self, parent)
         self.model = model
-        self.setWindowModality(Qt.WindowModal)
-        self.resize(700, 400)
+        if parent is not None:
+            self.setWindowModality(Qt.WindowModal)
+        self.setAttribute(Qt.WA_MacMetalStyle)
 
         # Widgets
         self.title_label = QtGui.QLabel()
@@ -67,6 +71,11 @@ class MergeView(QtGui.QDialog):
         self.checkbox_squash = QtGui.QCheckBox(self)
         self.checkbox_squash.setText(N_('Squash'))
 
+        self.checkbox_noff = QtGui.QCheckBox(self)
+        self.checkbox_noff.setText(N_('No fast forward'))
+        self.checkbox_noff.setChecked(False)
+        self.checkbox_noff_state = False
+
         self.checkbox_commit = QtGui.QCheckBox(self)
         self.checkbox_commit.setText(N_('Commit'))
         self.checkbox_commit.setChecked(True)
@@ -95,6 +104,7 @@ class MergeView(QtGui.QDialog):
         self.buttonlayt.addWidget(self.button_viz)
         self.buttonlayt.addStretch()
         self.buttonlayt.addWidget(self.checkbox_squash)
+        self.buttonlayt.addWidget(self.checkbox_noff)
         self.buttonlayt.addWidget(self.checkbox_commit)
         self.buttonlayt.addWidget(self.button_cancel)
         self.buttonlayt.addWidget(self.button_merge)
@@ -128,6 +138,7 @@ class MergeView(QtGui.QDialog):
         # Observer messages
         model.add_observer(model.message_updated, self.update_all)
         self.update_all()
+        self.resize(700, 400)
 
     def update_all(self):
         """Set the branch name for the window title and label."""
@@ -136,7 +147,7 @@ class MergeView(QtGui.QDialog):
 
     def update_title(self, dummy_txt=None):
         branch = self.model.currentbranch
-        revision = unicode(self.revision.text())
+        revision = ustr(self.revision.text())
         if revision:
             txt = (N_('Merge "%(revision)s" into "%(branch)s"') %
                    dict(revision=revision, branch=branch))
@@ -152,7 +163,14 @@ class MergeView(QtGui.QDialog):
                 self.checkbox_commit.checkState()
             self.checkbox_commit.setCheckState(Qt.Unchecked)
             self.checkbox_commit.setDisabled(True)
+            self.checkbox_noff_state =\
+                self.checkbox_noff.checkState()
+            self.checkbox_noff.setCheckState(Qt.Unchecked)
+            self.checkbox_noff.setDisabled(True)
         else:
+            self.checkbox_noff.setDisabled(False)
+            oldstateff = self.checkbox_noff_state
+            self.checkbox_noff.setCheckState(oldstateff)
             self.checkbox_commit.setDisabled(False)
             oldstate = self.checkbox_commit_state
             self.checkbox_commit.setCheckState(oldstate)
@@ -183,7 +201,7 @@ class MergeView(QtGui.QDialog):
 
     def viz_revision(self):
         """Launch a gitk-like viewer on the selection revision"""
-        revision = unicode(self.revision.text())
+        revision = ustr(self.revision.text())
         if not revision:
             qtutils.information(N_('No Revision Specified'),
                                 N_('You must specify a revision to view.'))
@@ -192,13 +210,14 @@ class MergeView(QtGui.QDialog):
 
     def merge_revision(self):
         """Merge the selected revision/branch"""
-        revision = unicode(self.revision.text())
+        revision = ustr(self.revision.text())
         if not revision:
             qtutils.information(N_('No Revision Specified'),
                                 N_('You must specify a revision to merge.'))
             return
 
+        noff = self.checkbox_noff.isChecked()
         do_commit = self.checkbox_commit.isChecked()
         squash = self.checkbox_squash.isChecked()
-        cmds.do(cmds.Merge, revision, not(do_commit), squash)
+        cmds.do(cmds.Merge, revision, not(do_commit), squash, noff)
         self.accept()
