@@ -1,5 +1,8 @@
+from __future__ import division, absolute_import, unicode_literals
+
 from PyQt4 import QtGui
 from PyQt4 import QtCore
+from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
 from cola import cmds
@@ -12,7 +15,7 @@ from cola.widgets.browse import GitTreeWidget
 from cola.widgets.browse import GitFileTreeModel
 
 
-def browse_recent():
+def browse_recent_files():
     parent = qtutils.active_window()
     dialog = RecentFileDialog(parent)
     dialog.resize(parent.width(), min(parent.height(), 420))
@@ -34,7 +37,7 @@ class RecentFileDialog(standard.Dialog):
     def __init__(self, parent):
         standard.Dialog.__init__(self, parent)
         self.setWindowTitle(N_('Recently Modified Files'))
-        self.setWindowModality(QtCore.Qt.WindowModal)
+        self.setWindowModality(Qt.WindowModal)
 
         count = 8
         self.update_thread = UpdateFileListThread(count)
@@ -53,16 +56,16 @@ class RecentFileDialog(standard.Dialog):
         self.refresh_button.setIcon(qtutils.reload_icon())
         self.refresh_button.setEnabled(False)
 
-        self.tree = GitTreeWidget(None, parent=self)
+        self.tree = GitTreeWidget(parent=self)
         self.tree_model = GitFileTreeModel(self)
         self.tree.setModel(self.tree_model)
 
         self.expand_button = QtGui.QPushButton()
-        self.expand_button.setText(N_('Expand'))
+        self.expand_button.setText(N_('Expand all'))
         self.expand_button.setIcon(qtutils.open_icon())
 
         self.collapse_button = QtGui.QPushButton()
-        self.collapse_button.setText(N_('Collapse'))
+        self.collapse_button.setText(N_('Collapse all'))
         self.collapse_button.setIcon(qtutils.dir_close_icon())
 
         self.edit_button = QtGui.QPushButton()
@@ -75,7 +78,7 @@ class RecentFileDialog(standard.Dialog):
         self.close_button.setText(N_('Close'))
 
         toplayout = QtGui.QHBoxLayout()
-        toplayout.setMargin(0)
+        toplayout.setMargin(defs.no_margin)
         toplayout.setSpacing(defs.spacing)
         toplayout.addWidget(self.count_label)
         toplayout.addWidget(self.count)
@@ -83,7 +86,7 @@ class RecentFileDialog(standard.Dialog):
         toplayout.addWidget(self.refresh_button)
 
         btnlayout = QtGui.QHBoxLayout()
-        btnlayout.setMargin(0)
+        btnlayout.setMargin(defs.no_margin)
         btnlayout.setSpacing(defs.spacing)
         btnlayout.addWidget(self.expand_button)
         btnlayout.addWidget(self.collapse_button)
