@@ -1,5 +1,6 @@
 """A widget for searching git commits"""
-import os
+from __future__ import division, absolute_import, unicode_literals
+
 import time
 import subprocess
 
@@ -7,6 +8,7 @@ from PyQt4 import QtGui
 from PyQt4 import QtCore
 from PyQt4.QtCore import SIGNAL
 
+from cola import core
 from cola import gitcmds
 from cola import utils
 from cola import qtutils
@@ -20,6 +22,7 @@ from cola.qtutils import dir_icon
 from cola.widgets import defs
 from cola.widgets import standard
 from cola.widgets.diff import DiffTextEdit
+from cola.compat import ustr
 
 
 def mkdate(timespec):
@@ -82,7 +85,7 @@ class SearchWidget(standard.Dialog):
         self.button_close.setText(N_('Close'))
 
         self.top_layout = QtGui.QHBoxLayout()
-        self.top_layout.setMargin(0)
+        self.top_layout.setMargin(defs.no_margin)
         self.top_layout.setSpacing(defs.button_spacing)
 
         self.top_layout.addWidget(self.query)
@@ -102,7 +105,7 @@ class SearchWidget(standard.Dialog):
         self.splitter.addWidget(self.commit_text)
 
         self.bottom_layout = QtGui.QHBoxLayout()
-        self.bottom_layout.setMargin(0)
+        self.bottom_layout.setMargin(defs.no_margin)
         self.bottom_layout.setSpacing(defs.spacing)
         self.bottom_layout.addWidget(self.button_export)
         self.bottom_layout.addWidget(self.button_cherrypick)
@@ -297,11 +300,11 @@ class Search(SearchWidget):
         self.end_date.setVisible(date_shown)
 
     def mode(self):
-        return str(self.mode_combo.currentText())
+        return ustr(self.mode_combo.currentText())
 
     def search_callback(self, *args):
         engineclass = self.engines[self.mode()]
-        self.model.query = unicode(self.query.text())
+        self.model.query = ustr(self.query.text())
         self.model.max_count = self.max_count.value()
 
         fmt = QtCore.Qt.ISODate
@@ -321,9 +324,9 @@ class Search(SearchWidget):
         if not paths:
             return
         filepaths = []
-        lenprefix = len(os.getcwd()) + 1
-        for path in map(lambda x: unicode(x), paths):
-            if not path.startswith(os.getcwd()):
+        lenprefix = len(core.getcwd()) + 1
+        for path in map(lambda x: ustr(x), paths):
+            if not path.startswith(core.getcwd()):
                 continue
             filepaths.append(path[lenprefix:])
         query = subprocess.list2cmdline(filepaths)
