@@ -1,5 +1,8 @@
 from __future__ import division, absolute_import, unicode_literals
 
+import time
+
+
 from PyQt4 import QtGui
 from PyQt4 import QtCore
 from PyQt4.QtCore import Qt
@@ -9,7 +12,8 @@ from PyQt4.QtGui import QDockWidget
 from cola import core
 from cola import gitcfg
 from cola import qtcompat
-from cola import settings
+from cola import qtutils
+from cola.settings import Settings
 
 
 class WidgetMixin(object):
@@ -37,14 +41,18 @@ class WidgetMixin(object):
         """Returns the name of the view class"""
         return self.__class__.__name__.lower()
 
-    def save_state(self, handler=None):
-        if handler is None:
-            handler = settings.Settings()
+    def save_state(self, settings=None):
+        if settings is None:
+            settings = Settings()
+            settings.load()
         if gitcfg.instance().get('cola.savewindowsettings', True):
-            handler.save_gui_state(self)
+            settings.save_gui_state(self)
 
-    def restore_state(self):
-        state = settings.Settings().get_gui_state(self)
+    def restore_state(self, settings=None):
+        if settings is None:
+            settings = Settings()
+            settings.load()
+        state = settings.get_gui_state(self)
         return bool(state) and self.apply_state(state)
 
     def apply_state(self, state):
@@ -79,9 +87,10 @@ class WidgetMixin(object):
         }
 
     def closeEvent(self, event):
-        s = settings.Settings()
-        s.add_recent(core.getcwd())
-        self.save_state(handler=s)
+        settings = Settings()
+        settings.load()
+        settings.add_recent(core.getcwd())
+        self.save_state(settings=settings)
         self.QtClass.closeEvent(self, event)
 
 
@@ -341,3 +350,83 @@ class DraggableTreeWidget(DraggableTreeMixin, QtGui.QTreeWidget):
     def __init__(self, parent=None):
         QtGui.QTreeWidget.__init__(self, parent)
         DraggableTreeMixin.__init__(self, QtGui.QTreeWidget)
+
+
+class ProgressDialog(QtGui.QProgressDialog):
+    """Custom progress dialog
+
+    This dialog ignores the ESC key so that it is not
+    prematurely closed.
+
+    An thread is spawned to animate the progress label text.
+
+    """
+    def __init__(self, title, label, parent):
+        QtGui.QProgressDialog.__init__(self, parent)
+        self.setFont(qtutils.diff_font())
+        self.setRange(0, 0)
+        self.setCancelButton(None)
+        if parent is not None:
+            self.setWindowModality(Qt.WindowModal)
+        self.progress_thread = ProgressAnimationThread(label, self)
+        self.connect(self.progress_thread,
+                     SIGNAL('update_progress'), self.update_progress)
+
+        self.set_details(title, label)
+
+    def set_details(self, title, label):
+        self.setWindowTitle(title)
+        self.setLabelText(label + '     ')
+        self.progress_thread.set_text(label)
+
+    def update_progress(self, txt):
+        self.setLabelText(txt)
+
+    def keyPressEvent(self, event):
+        if event.key() != Qt.Key_Escape:
+            QtGui.QProgressDialog.keyPressEvent(self, event)
+
+    def show(self):
+        QtGui.QApplication.setOverrideCursor(Qt.WaitCursor)
+        self.progress_thread.start()
+        QtGui.QProgressDialog.show(self)
+
+    def hide(self):
+        QtGui.QApplication.restoreOverrideCursor()
+        self.progress_thread.stop()
+        self.progress_thread.wait()
+        QtGui.QProgressDialog.hide(self)
+
+
+class ProgressAnimationThread(QtCore.QThread):
+    """Emits a pseudo-animated text stream for progress bars"""
+
+    def __init__(self, txt, parent, timeout=0.1):
+        QtCore.QThread.__init__(self, parent)
+        self.running = False
+        self.txt = txt
+        self.timeout = timeout
+        self.symbols = [
+            '.  ..',
+            '..  .',
+            '...  ',
+            ' ... ',
+            '  ...',
+        ]
+        self.idx = -1
+
+    def set_text(self, txt):
+        self.txt = txt
+
+    def next(self):
+        self.idx = (self.idx + 1) % len(self.symbols)
+        return self.txt + self.symbols[self.idx]
+
+    def stop(self):
+        self.running = False
+
+    def run(self):
+        self.running = True
+        while self.running:
+            self.emit(SIGNAL('update_progress'), self.next())
+            time.sleep(self.timeout)
