@@ -1,4 +1,5 @@
-import os
+from __future__ import division, absolute_import, unicode_literals
+
 import subprocess
 import itertools
 
@@ -11,11 +12,12 @@ from cola import cmds
 from cola import core
 from cola import qtutils
 from cola import utils
-from cola.compat import set
 from cola.i18n import N_
 from cola.interaction import Interaction
 from cola.models import main
 from cola.models import selection
+from cola.widgets import completion
+from cola.widgets import defs
 
 
 class StatusWidget(QtGui.QWidget):
@@ -26,14 +28,40 @@ class StatusWidget(QtGui.QWidget):
     Qt signals.
 
     """
-    def __init__(self, parent=None):
+    def __init__(self, titlebar, parent=None):
         QtGui.QWidget.__init__(self, parent)
-        self.layout = QtGui.QVBoxLayout(self)
-        self.setLayout(self.layout)
+
+        tooltip = N_('Toggle the paths filter')
+        self.filter_button = qtutils.create_action_button(
+                tooltip=tooltip,
+                icon=qtutils.filter_icon())
+
+        self.filter_widget = StatusFilterWidget(self)
+        self.filter_widget.hide()
 
         self.tree = StatusTreeWidget(self)
-        self.layout.addWidget(self.tree)
-        self.layout.setContentsMargins(0, 0, 0, 0)
+
+        self.main_layout = QtGui.QVBoxLayout(self)
+        self.main_layout.setMargin(defs.no_margin)
+        self.main_layout.setSpacing(defs.no_spacing)
+
+        self.main_layout.addWidget(self.filter_widget)
+        self.main_layout.addWidget(self.tree)
+        self.setLayout(self.main_layout)
+
+        self.toggle_action = qtutils.add_action(self, tooltip,
+                self.toggle_filter, 'Shift+Ctrl+F')
+
+        titlebar.add_corner_widget(self.filter_button)
+        qtutils.connect_button(self.filter_button, self.toggle_filter)
+
+    def toggle_filter(self):
+        shown = not self.filter_widget.isVisible()
+        self.filter_widget.setVisible(shown)
+        if shown:
+            self.filter_widget.setFocus(True)
+        else:
+            self.tree.setFocus(True)
 
     def set_initial_size(self):
         self.setMaximumWidth(222)
@@ -45,6 +73,11 @@ class StatusWidget(QtGui.QWidget):
     def refresh(self):
         self.tree.show_selection()
 
+    def set_filter(self, txt):
+        self.filter_widget.setVisible(True)
+        self.filter_widget.text.set_value(txt)
+        self.filter_widget.apply_filter()
+
 
 class StatusTreeWidget(QtGui.QTreeWidget):
     # Item categories
@@ -69,6 +102,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.setAnimated(True)
         self.setRootIsDecorated(False)
         self.setIndentation(0)
+        self.setDragEnabled(True)
 
         self.add_item(N_('Staged'), hide=True)
         self.add_item(N_('Unmerged'), hide=True)
@@ -82,22 +116,28 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.old_current_item = None
         self.expanded_items = set()
 
-        self.process_selection = qtutils.add_action(self,
+        self.process_selection_action = qtutils.add_action(self,
                 N_('Stage / Unstage'), self._process_selection,
                 cmds.Stage.SHORTCUT)
 
-        self.launch_difftool = qtutils.add_action(self,
+        self.revert_unstaged_edits_action = qtutils.add_action(self,
+                N_('Revert Unstaged Edits...'),
+                cmds.run(cmds.RevertUnstagedEdits),
+                cmds.RevertUnstagedEdits.SHORTCUT)
+        self.revert_unstaged_edits_action.setIcon(qtutils.icon('undo.svg'))
+
+        self.launch_difftool_action = qtutils.add_action(self,
                 cmds.LaunchDifftool.name(),
                 cmds.run(cmds.LaunchDifftool),
                 cmds.LaunchDifftool.SHORTCUT)
-        self.launch_difftool.setIcon(qtutils.icon('git.svg'))
+        self.launch_difftool_action.setIcon(qtutils.icon('git.svg'))
 
-        self.launch_editor = qtutils.add_action(self,
+        self.launch_editor_action = qtutils.add_action(self,
                 cmds.LaunchEditor.name(),
                 cmds.run(cmds.LaunchEditor),
                 cmds.LaunchEditor.SHORTCUT,
                 'Return', 'Enter')
-        self.launch_editor.setIcon(qtutils.options_icon())
+        self.launch_editor_action.setIcon(qtutils.options_icon())
 
         if not utils.is_win32():
             self.open_using_default_app = qtutils.add_action(self,
@@ -106,23 +146,38 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                     cmds.OpenDefaultApp.SHORTCUT)
             self.open_using_default_app.setIcon(qtutils.file_icon())
 
-            self.open_parent_dir = qtutils.add_action(self,
+            self.open_parent_dir_action = qtutils.add_action(self,
                     cmds.OpenParentDir.name(),
                     self._open_parent_dir,
                     cmds.OpenParentDir.SHORTCUT)
-            self.open_parent_dir.setIcon(qtutils.open_file_icon())
+            self.open_parent_dir_action.setIcon(qtutils.open_file_icon())
 
-        self.up = qtutils.add_action(self, N_('Move Up'), self.move_up,
-                                     Qt.Key_K)
+        self.up_action = qtutils.add_action(self,
+                N_('Move Up'), self.move_up, Qt.Key_K)
 
-        self.down = qtutils.add_action(self, N_('Move Down'), self.move_down,
-                                       Qt.Key_J)
+        self.down_action = qtutils.add_action(self,
+                N_('Move Down'), self.move_down, Qt.Key_J)
 
         self.copy_path_action = qtutils.add_action(self,
                 N_('Copy Path to Clipboard'),
                 self.copy_path, QtGui.QKeySequence.Copy)
         self.copy_path_action.setIcon(qtutils.theme_icon('edit-copy.svg'))
 
+        if cmds.MoveToTrash.AVAILABLE:
+            self.move_to_trash_action = qtutils.add_action(self,
+                    N_('Move file(s) to trash'),
+                    self._trash_untracked_files, cmds.MoveToTrash.SHORTCUT)
+            self.move_to_trash_action.setIcon(qtutils.discard_icon())
+            delete_shortcut = cmds.Delete.SHORTCUT
+        else:
+            self.move_to_trash_action = None
+            delete_shortcut = cmds.Delete.ALT_SHORTCUT
+
+        self.delete_untracked_files_action = qtutils.add_action(self,
+                N_('Delete File(s)...'),
+                self._delete_untracked_files, delete_shortcut)
+        self.delete_untracked_files_action.setIcon(qtutils.discard_icon())
+
         self.connect(self, SIGNAL('about_to_update'), self._about_to_update)
         self.connect(self, SIGNAL('updated'), self._updated)
 
@@ -148,7 +203,6 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         # TODO no icon
         font = self.font()
         font.setBold(True)
-        font.setCapitalization(QtGui.QFont.SmallCaps)
 
         item = QtGui.QTreeWidgetItem(self)
         item.setFont(0, font)
@@ -217,13 +271,13 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         # we do not need to rerun the callbacks which were triggered
         # above.  Block signals to skip the callbacks.
         self.blockSignals(True)
-        for (new, old, selection, reselect) in saved_selection:
-            for item in selection:
+        for (new, old, sel, reselect) in saved_selection:
+            for item in sel:
                 if item in new:
                     reselect(item, current=False)
         self.blockSignals(False)
 
-        for (new, old, selection, reselect) in saved_selection:
+        for (new, old, sel, reselect) in saved_selection:
             # When modified is staged, select the next modified item
             # When unmerged is staged, select the next unmerged item
             # When unstaging, select the next staged item
@@ -231,7 +285,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             if len(new) >= len(old):
                 # The list did not shrink so it is not one of these cases.
                 continue
-            for item in selection:
+            for item in sel:
                 # The item still exists so ignore it
                 if item in new or item not in old:
                     continue
@@ -330,6 +384,13 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.restore_selection()
         self.restore_scrollbar()
         self.update_column_widths()
+        self.update_actions()
+
+    def update_actions(self, selected=None):
+        if selected is None:
+            selected = selection.selection()
+        can_revert_unstaged_edits = bool(selected.staged or selected.modified)
+        self.revert_unstaged_edits_action.setEnabled(can_revert_unstaged_edits)
 
     def set_staged(self, items):
         """Adds items to the 'Staged' subtree."""
@@ -454,10 +515,15 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                                     cmds.run(cmds.Unstage, self.staged()))
             action.setShortcut(cmds.Unstage.SHORTCUT)
 
-        menu.addAction(self.launch_editor)
-        menu.addAction(self.launch_difftool)
+        # Do all of the selected items exist?
+        staged_items = self.staged_items()
+        all_exist = all([i.exists for i in staged_items])
 
-        if not utils.is_win32():
+        if all_exist:
+            menu.addAction(self.launch_editor_action)
+            menu.addAction(self.launch_difftool_action)
+
+        if all_exist and not utils.is_win32():
             menu.addSeparator()
             action = menu.addAction(qtutils.file_icon(),
                     cmds.OpenDefaultApp.name(),
@@ -471,9 +537,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
 
         if self.m.undoable():
             menu.addSeparator()
-            menu.addAction(qtutils.icon('undo.svg'),
-                           N_('Revert Unstaged Edits...'),
-                           lambda: self._revert_unstaged_edits(staged=True))
+            menu.addAction(self.revert_unstaged_edits_action)
             menu.addAction(qtutils.icon('undo.svg'),
                            N_('Revert Uncommited Edits...'),
                            lambda: self._revert_uncommitted_edits(
@@ -488,7 +552,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                        cmds.run(cmds.OpenRepo,
                                 core.abspath(s.staged[0])))
 
-        menu.addAction(self.launch_editor)
+        menu.addAction(self.launch_editor_action)
         menu.addSeparator()
 
         action = menu.addAction(qtutils.icon('remove.svg'),
@@ -501,14 +565,14 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         return menu
 
     def _create_unmerged_context_menu(self, menu, s):
-        menu.addAction(self.launch_difftool)
+        menu.addAction(self.launch_difftool_action)
 
         action = menu.addAction(qtutils.icon('add.svg'),
                                 N_('Stage Selected'),
                                 cmds.run(cmds.Stage, self.unstaged()))
         action.setShortcut(cmds.Stage.SHORTCUT)
         menu.addSeparator()
-        menu.addAction(self.launch_editor)
+        menu.addAction(self.launch_editor_action)
 
         if not utils.is_win32():
             menu.addSeparator()
@@ -538,24 +602,26 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                                     cmds.run(cmds.Stage, self.unstaged()))
             action.setShortcut(cmds.Stage.SHORTCUT)
 
-        if self.unstaged():
-            menu.addAction(self.launch_editor)
+        # Do all of the selected items exist?
+        unstaged_items = self.unstaged_items()
+        all_exist = all([i.exists for i in unstaged_items])
 
-        if s.modified and self.m.stageable():
-            menu.addAction(self.launch_difftool)
+        if all_exist and self.unstaged():
+            menu.addAction(self.launch_editor_action)
+
+        if all_exist and s.modified and self.m.stageable():
+            menu.addAction(self.launch_difftool_action)
 
         if s.modified and self.m.stageable():
             if self.m.undoable():
                 menu.addSeparator()
-                menu.addAction(qtutils.icon('undo.svg'),
-                               N_('Revert Unstaged Edits...'),
-                               self._revert_unstaged_edits)
+                menu.addAction(self.revert_unstaged_edits_action)
                 menu.addAction(qtutils.icon('undo.svg'),
                                N_('Revert Uncommited Edits...'),
                                lambda: self._revert_uncommitted_edits(
                                             self.modified()))
 
-        if self.unstaged() and not utils.is_win32():
+        if all_exist and self.unstaged() and not utils.is_win32():
             menu.addSeparator()
             action = menu.addAction(qtutils.file_icon(),
                     cmds.OpenDefaultApp.name(),
@@ -567,10 +633,11 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                     self._open_parent_dir)
             action.setShortcut(cmds.OpenParentDir.SHORTCUT)
 
-        if s.untracked:
+        if all_exist and s.untracked:
             menu.addSeparator()
-            menu.addAction(qtutils.discard_icon(),
-                           N_('Delete File(s)...'), self._delete_files)
+            if self.move_to_trash_action is not None:
+                menu.addAction(self.move_to_trash_action)
+            menu.addAction(self.delete_untracked_files_action)
             menu.addSeparator()
             menu.addAction(qtutils.icon('edit-clear.svg'),
                            N_('Add to .gitignore'),
@@ -585,7 +652,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                        N_('Launch git-cola'),
                        cmds.run(cmds.OpenRepo, core.abspath(s.modified[0])))
 
-        menu.addAction(self.launch_editor)
+        menu.addAction(self.launch_editor_action)
 
         if self.m.stageable():
             menu.addSeparator()
@@ -599,52 +666,11 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         return menu
 
 
-    def _delete_files(self):
-        files = self.untracked()
-        count = len(files)
-        if count == 0:
-            return
+    def _delete_untracked_files(self):
+        cmds.do(cmds.Delete, self.untracked())
 
-        title = N_('Delete Files?')
-        msg = N_('The following files will be deleted:') + '\n\n'
-
-        fileinfo = subprocess.list2cmdline(files)
-        if len(fileinfo) > 2048:
-            fileinfo = fileinfo[:2048].rstrip() + '...'
-        msg += fileinfo
-
-        info_txt = N_('Delete %d file(s)?') % count
-        ok_txt = N_('Delete Files')
-
-        if qtutils.confirm(title, msg, info_txt, ok_txt,
-                           default=True,
-                           icon=qtutils.discard_icon()):
-            cmds.do(cmds.Delete, files)
-
-    def _revert_unstaged_edits(self, staged=False):
-        if not self.m.undoable():
-            return
-        if staged:
-            items_to_undo = self.staged()
-        else:
-            items_to_undo = self.modified()
-
-        if items_to_undo:
-            if not qtutils.confirm(N_('Revert Unstaged Changes?'),
-                                   N_('This operation drops unstaged changes.\n'
-                                      'These changes cannot be recovered.'),
-                                   N_('Revert the unstaged changes?'),
-                                   N_('Revert Unstaged Changes'),
-                                   default=True,
-                                   icon=qtutils.icon('undo.svg')):
-                return
-            args = []
-            if not staged and self.m.amending():
-                args.append(self.m.head)
-            cmds.do(cmds.Checkout, args + ['--'] + items_to_undo)
-        else:
-            msg = N_('No files selected for checkout from HEAD.')
-            Interaction.log(msg)
+    def _trash_untracked_files(self):
+        cmds.do(cmds.MoveToTrash, self.untracked())
 
     def _revert_uncommitted_edits(self, items_to_undo):
         if items_to_undo:
@@ -761,10 +787,30 @@ class StatusTreeWidget(QtGui.QTreeWidget):
     def untracked(self):
         return self._subtree_selection(self.idx_untracked, self.m.untracked)
 
+    def staged_items(self):
+        return self._subtree_selection_items(self.idx_staged)
+
+    def unstaged_items(self):
+        return (self.unmerged_items() + self.modified_items() +
+                self.untracked_items())
+
+    def modified_items(self):
+        return self._subtree_selection_items(self.idx_modified)
+
+    def unmerged_items(self):
+        return self._subtree_selection_items(self.idx_unmerged)
+
+    def untracked_items(self):
+        return self._subtree_selection_items(self.idx_untracked)
+
     def _subtree_selection(self, idx, items):
         item = self.topLevelItem(idx)
         return qtutils.tree_selection(item, items)
 
+    def _subtree_selection_items(self, idx):
+        item = self.topLevelItem(idx)
+        return qtutils.tree_selection_items(item)
+
     def double_clicked(self, item, idx):
         """Called when an item is double-clicked in the repo status tree."""
         self._process_selection()
@@ -793,7 +839,10 @@ class StatusTreeWidget(QtGui.QTreeWidget):
     def show_selection(self):
         """Show the selected item."""
         # Sync the selection model
-        selection.selection_model().set_selection(self.selection())
+        selected = self.selection()
+        selection.selection_model().set_selection(selected)
+        self.update_actions(selected=selected)
+
         selected_indexes = self.selected_indexes()
         if not selected_indexes:
             if self.m.amending():
@@ -869,6 +918,36 @@ class StatusTreeWidget(QtGui.QTreeWidget):
     def copy_path(self):
         """Copy a selected path to the clipboard"""
         filename = selection.selection_model().filename()
-        if filename is not None:
-            curdir = os.getcwdu()
-            qtutils.set_clipboard(os.path.join(curdir, filename))
+        qtutils.copy_path(filename)
+
+    def mimeData(self, items):
+        """Return a list of absolute-path URLs"""
+        paths = qtutils.paths_from_items(items, item_filter=lambda x: x.exists)
+        return qtutils.mimedata_from_paths(paths)
+
+
+class StatusFilterWidget(QtGui.QWidget):
+
+    def __init__(self, parent):
+        QtGui.QWidget.__init__(self,parent)
+        self.main_model = main.model()
+
+        hint = N_('Filter paths...')
+        self.text = completion.GitStatusFilterLineEdit(hint=hint, parent=self)
+        self.text.setToolTip(hint)
+        self.text.enable_hint(True)
+        self.setFocusProxy(self.text)
+
+        self.main_layout = QtGui.QHBoxLayout()
+        self.main_layout.setMargin(defs.no_margin)
+        self.main_layout.setSpacing(defs.spacing)
+        self.main_layout.addWidget(self.text)
+        self.setLayout(self.main_layout)
+
+        self.connect(self.text, SIGNAL('changed()'), self.apply_filter)
+        self.connect(self.text, SIGNAL('returnPressed()'), self.apply_filter)
+
+    def apply_filter(self):
+        text = self.text.value()
+        paths = utils.shell_split(text)
+        self.main_model.update_path_filter(paths)
