@@ -1,21 +1,23 @@
 """Covers interfaces used by the classic view."""
-import os
+from __future__ import unicode_literals
 
-import helper
+from cola import core
 from cola import gitcmds
 from cola.models.main import MainModel
 
+from test import helper
+
 
 class ClassicModelTestCase(helper.GitRepositoryTestCase):
     """Tests interfaces used by the classic view."""
 
     def setUp(self):
         helper.GitRepositoryTestCase.setUp(self, commit=False)
-        self.model = MainModel(cwd=os.getcwd())
+        self.model = MainModel(cwd=core.getcwd())
 
     def test_everything(self):
         """Test the MainModel.everything() method."""
-        self.shell('touch other-file')
+        self.touch('other-file')
         everything = self.model.everything()
 
         self.assertTrue('A' in everything)
@@ -24,10 +26,8 @@ class ClassicModelTestCase(helper.GitRepositoryTestCase):
 
     def test_stage_paths_untracked(self):
         """Test stage_paths() with an untracked file."""
-        self.shell("""
-            mkdir -p foo/bar &&
-            touch foo/bar/baz
-        """)
+        core.makedirs('foo/bar')
+        self.touch('foo/bar/baz')
         self.model.stage_paths(['foo'])
 
         self.assertTrue('foo/bar/baz' in self.model.staged)
@@ -36,11 +36,9 @@ class ClassicModelTestCase(helper.GitRepositoryTestCase):
 
     def test_unstage_paths(self):
         """Test a simple usage of unstage_paths()."""
-        self.shell("""
-            git commit -m'initial commit' > /dev/null
-            echo change > A &&
-            git add A
-        """)
+        self.commit_files()
+        self.write_file('A', 'change')
+        self.git('add', 'A')
         gitcmds.unstage_paths(['A'])
         self.model.update_status()
 
@@ -57,12 +55,10 @@ class ClassicModelTestCase(helper.GitRepositoryTestCase):
 
     def test_unstage_paths_subdir(self):
         """Test unstage_paths() in a subdirectory."""
-        self.shell("git commit -m'initial commit' > /dev/null")
-        self.shell("""
-            mkdir -p foo/bar &&
-            touch foo/bar/baz &&
-            git add foo/bar/baz
-        """)
+        self.git('commit', '-m', 'intitial commit')
+        core.makedirs('foo/bar')
+        self.touch('foo/bar/baz')
+        self.git('add', 'foo/bar/baz')
         gitcmds.unstage_paths(['foo'])
         self.model.update_status()
 
