@@ -2,12 +2,14 @@
 """Tests basic git operations: commit, log, config"""
 from __future__ import unicode_literals
 
-import os
 import unittest
 
-import helper
+from cola import core
 from cola.models.main import MainModel
 
+from test import helper
+
+
 class ColaBasicGitTestCase(helper.GitRepositoryTestCase):
 
     def setUp(self):
@@ -15,22 +17,20 @@ class ColaBasicGitTestCase(helper.GitRepositoryTestCase):
 
     def test_git_commit(self):
         """Test running 'git commit' via cola.git"""
-        self.shell("""
-            echo A > A
-            echo B > B
-            git add A B
-            """)
+        self.write_file('A', 'A')
+        self.write_file('B', 'B')
+        self.git('add', 'A', 'B')
 
-        model = MainModel(cwd=os.getcwd())
+        model = MainModel(cwd=core.getcwd())
         model.git.commit(m='commit test')
-        log = helper.pipe('git log --pretty=oneline | wc -l')
+        log = self.git('log', '--pretty=oneline')
 
-        self.assertEqual(log.strip(), '1')
+        self.assertEqual(len(log.splitlines()), 1)
 
     def test_git_config(self):
         """Test cola.git.config()"""
-        self.shell('git config section.key value')
-        model = MainModel(cwd=os.getcwd())
+        self.git('config', 'section.key', 'value')
+        model = MainModel(cwd=core.getcwd())
         value = model.git.config('section.key', get=True)
 
         self.assertEqual(value, (0, 'value', ''))
