@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+from __future__ import unicode_literals
 
 import unittest
 from cola import textwrap
@@ -74,7 +75,7 @@ else:
         text = """
 This test is so meta, even this sentence
 
-With-special-tag: Avoids word-wrap
+Cheered-on-by: Avoids word-wrap
 """
 
         expect = """
@@ -87,7 +88,7 @@ even
 this
 sentence
 
-With-special-tag: Avoids word-wrap
+Cheered-on-by: Avoids word-wrap
 """
 
         self.assertEqual(self.wrap(text), expect)
